from __future__ import annotations

from hashlib import sha256

import datasets

from mteb.abstasks.MultilingualTask import MultilingualTask
from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskRetrieval import AbsTaskRetrieval

_LANGUAGES = {
    "as": ["asm-Beng"],
    "bn": ["ben-Beng"],
    "gu": ["guj-Gujr"],
    "hi": ["hin-Deva"],
    "kn": ["kan-Knda"],
    "ml": ["mal-Mlym"],
    "mr": ["mar-Deva"],
    "or": ["ory-Orya"],
    "pa": ["pan-Guru"],
    "ta": ["tam-Taml"],
    "te": ["tel-Telu"],
}


class IndicQARetrieval(MultilingualTask, AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="IndicQARetrieval",
        dataset={
            "path": "ai4bharat/IndicQA",
            "revision": "570d90ae4f7b64fe4fdd5f42fc9f9279b8c9fd9d",
            "trust_remote_code": True,
        },
        description="IndicQA is a manually curated cloze-style reading comprehension dataset that can be used for evaluating question-answering models in 11 Indic languages. It is repurposed retrieving relevant context for each question.",
        reference="https://arxiv.org/abs/2212.05409",
        type="Retrieval",
        category="s2p",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=_LANGUAGES,
        main_score="ndcg_at_10",
        date=("2022-08-01", "2022-12-20"),
        domains=["Web", "Written"],
        task_subtypes=[],
        license="cc0-1.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="machine-translated and verified",
        bibtex_citation="""@article{doddapaneni2022towards,
  title     = {Towards Leaving No Indic Language Behind: Building Monolingual Corpora, Benchmark and Models for Indic Languages},
  author    = {Sumanth Doddapaneni and Rahul Aralikatte and Gowtham Ramesh and Shreyansh Goyal and Mitesh M. Khapra and Anoop Kunchukuttan and Pratyush Kumar},
  journal   = {Annual Meeting of the Association for Computational Linguistics},
  year      = {2022},
  doi       = {10.18653/v1/2023.acl-long.693}
}""",
    )

    def load_data(self, **kwargs):
        if self.data_loaded:
            return

        split = "test"
        queries = {lang: {split: {}} for lang in self.hf_subsets}
        corpus = {lang: {split: {}} for lang in self.hf_subsets}
        relevant_docs = {lang: {split: {}} for lang in self.hf_subsets}

        for lang in self.hf_subsets:
            data = datasets.load_dataset(
                name=f"indicqa.{lang}", **self.metadata_dict["dataset"]
            )[split]
            data = data.filter(lambda x: x["answers"]["text"] != "")

            question_ids = {
                question: sha256(question.encode("utf-8")).hexdigest()
                for question in set(data["question"])
            }
            context_ids = {
                context: sha256(context.encode("utf-8")).hexdigest()
                for context in set(data["context"])
            }

            for row in data:
                question = row["question"]
                context = row["context"]
                query_id = question_ids[question]
                queries[lang][split][query_id] = question

                doc_id = context_ids[context]
                corpus[lang][split][doc_id] = {"text": context}
                if query_id not in relevant_docs[lang][split]:
                    relevant_docs[lang][split][query_id] = {}
                relevant_docs[lang][split][query_id][doc_id] = 1

        self.corpus = datasets.DatasetDict(corpus)
        self.queries = datasets.DatasetDict(queries)
        self.relevant_docs = datasets.DatasetDict(relevant_docs)

        self.data_loaded = True



from __future__ import annotations

from mteb.abstasks.Image.AbsTaskImageClassification import AbsTaskImageClassification
from mteb.abstasks.TaskMetadata import TaskMetadata


class OxfordPetsClassification(AbsTaskImageClassification):
    metadata = TaskMetadata(
        name="OxfordPets",
        description="Classifying animal images.",
        reference="https://ieeexplore.ieee.org/abstract/document/6248092",
        dataset={
            "path": "isaacchung/OxfordPets",
            "revision": "557b480fae8d69247be74d9503b378a09425096f",
        },
        type="ImageClassification",
        category="i2i",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=(
            "2009-01-01",
            "2010-01-01",
        ),  # Estimated range for the collection of reviews
        domains=["Encyclopaedic"],
        task_subtypes=["Object recognition"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        modalities=["image"],
        sample_creation="created",
        bibtex_citation="""@INPROCEEDINGS{6248092,
  author={Parkhi, Omkar M and Vedaldi, Andrea and Zisserman, Andrew and Jawahar, C. V.},
  booktitle={2012 IEEE Conference on Computer Vision and Pattern Recognition}, 
  title={Cats and dogs}, 
  year={2012},
  volume={},
  number={},
  pages={3498-3505},
  keywords={Positron emission tomography;Image segmentation;Cats;Dogs;Layout;Deformable models;Head},
  doi={10.1109/CVPR.2012.6248092}}
        """,
        descriptive_stats={
            "n_samples": {"test": 3669},
            "avg_character_length": {"test": 431.4},
        },
    )



"""This script contains functions that are used to get an overview of the MTEB benchmark."""

from __future__ import annotations

import difflib
import logging
from collections import Counter

import pandas as pd

from mteb.abstasks.AbsTask import AbsTask
from mteb.abstasks.TaskMetadata import TASK_CATEGORY, TASK_DOMAIN, TASK_TYPE
from mteb.languages import (
    ISO_TO_LANGUAGE,
    ISO_TO_SCRIPT,
    path_to_lang_codes,
    path_to_lang_scripts,
)
from mteb.tasks import *  # import all tasks

logger = logging.getLogger(__name__)


# Create task registry


def create_task_list() -> list[type[AbsTask]]:
    tasks_categories_cls = list(AbsTask.__subclasses__())
    tasks = [
        cls
        for cat_cls in tasks_categories_cls
        for cls in cat_cls.__subclasses__()
        if cat_cls.__name__.startswith("AbsTask")
    ]
    return tasks


def create_name_to_task_mapping() -> dict[str, type[AbsTask]]:
    tasks = create_task_list()
    return {cls.metadata.name: cls for cls in tasks}


TASKS_REGISTRY = create_name_to_task_mapping()


def check_is_valid_script(script: str) -> None:
    if script not in ISO_TO_SCRIPT:
        raise ValueError(
            f"Invalid script code: {script}, you can find valid ISO 15924 codes in {path_to_lang_scripts}"
        )


def check_is_valid_language(lang: str) -> None:
    if lang not in ISO_TO_LANGUAGE:
        raise ValueError(
            f"Invalid language code: {lang}, you can find valid ISO 639-3 codes in {path_to_lang_codes}"
        )


def filter_superseded_datasets(tasks: list[AbsTask]) -> list[AbsTask]:
    return [t for t in tasks if t.superseded_by is None]


def filter_tasks_by_languages(
    tasks: list[AbsTask], languages: list[str]
) -> list[AbsTask]:
    [check_is_valid_language(lang) for lang in languages]
    langs_to_keep = set(languages)
    return [t for t in tasks if langs_to_keep.intersection(t.metadata.languages)]


def filter_tasks_by_script(tasks: list[AbsTask], script: list[str]) -> list[AbsTask]:
    [check_is_valid_script(s) for s in script]
    script_to_keep = set(script)
    return [t for t in tasks if script_to_keep.intersection(t.metadata.scripts)]


def filter_tasks_by_domains(
    tasks: list[AbsTask], domains: list[TASK_DOMAIN]
) -> list[AbsTask]:
    domains_to_keep = set(domains)

    def _convert_to_set(domain: list[TASK_DOMAIN] | None) -> set:
        return set(domain) if domain is not None else set()

    return [
        t
        for t in tasks
        if domains_to_keep.intersection(_convert_to_set(t.metadata.domains))
    ]


def filter_tasks_by_task_types(
    tasks: list[AbsTask], task_types: list[TASK_TYPE]
) -> list[AbsTask]:
    _task_types = set(task_types)
    return [t for t in tasks if t.metadata.type in _task_types]


def filter_task_by_categories(
    tasks: list[AbsTask], categories: list[TASK_CATEGORY]
) -> list[AbsTask]:
    _categories = set(categories)
    return [t for t in tasks if t.metadata.category in _categories]


class MTEBTasks(tuple):
    def __repr__(self) -> str:
        return "MTEBTasks" + super().__repr__()

    @staticmethod
    def _extract_property_from_task(task, property):
        if hasattr(task.metadata, property):
            return getattr(task.metadata, property)
        elif hasattr(task, property):
            return getattr(task, property)
        elif property in task.metadata_dict:
            return task.metadata_dict[property]
        else:
            raise KeyError("Property neither in Task attribute or in task metadata.")

    @property
    def languages(self) -> set:
        """Return all languages from tasks"""
        langs = set()
        for task in self:
            for lg in task.languages:
                langs.add(lg)
        return langs

    def count_languages(self) -> dict:
        """Summarize count of all languages from tasks"""
        langs = []
        for task in self:
            langs.extend(task.languages)
        return Counter(langs)

    def to_markdown(
        self, properties: list[str] = ["type", "license", "languages"]
    ) -> str:
        """Generate markdown table with tasks summary

        Args:
            properties: list of metadata to summarize from a Task class.

        Returns:
            string with a markdown table.
        """
        markdown_table = "| Task" + "".join([f"| {p} " for p in properties]) + "|\n"
        _head_sep = "| ---" * len(properties) + " |\n"
        markdown_table += _head_sep
        for task in self:
            markdown_table += f"| {task.metadata.name}"
            markdown_table += "".join(
                [f"| {self._extract_property_from_task(task, p)}" for p in properties]
            )
            markdown_table += " |\n"
        return markdown_table

    def to_dataframe(
        self,
        properties: list[str] = ["name", "type", "languages", "domains", "license"],
    ) -> pd.DataFrame:
        """Generate pandas DataFrame with tasks summary

        Args:
            properties: list of metadata to summarize from a Task class.

        Returns:
            pandas DataFrame.
        """
        data = []
        for task in self:
            data.append(
                {p: self._extract_property_from_task(task, p) for p in properties}
            )
        return pd.DataFrame(data)

    def to_latex(
        self,
        properties: list[str] = ["name", "type", "languages", "domains", "license"],
        group_indices: list[str] | None = ["type", "name"],
        include_citation_in_name: bool = True,
        limit_n_entries: int | None = 3,
    ) -> str:
        """Generate a LaTeX table of the tasks.

        Args:
            properties: list of metadata to summarize from a Task class.
            group_indices: list of properties to group the table by.
            include_citation_in_name: Whether to include the citation in the name.
            limit_n_entries: Limit the number of entries for cell values, e.g. number of languages and domains. Will use "..." to indicate that
                there are more entries.
        """
        if include_citation_in_name and "name" in properties:
            properties += ["intext_citation"]
            df = self.to_dataframe(properties)
            df["name"] = df["name"] + " " + df["intext_citation"]
            df = df.drop(columns=["intext_citation"])
        else:
            df = self.to_dataframe(properties)

        if limit_n_entries and df.shape[0]:  # ensure that there are entries
            for col in df.columns:
                # check if content is a list or set
                if isinstance(df[col].iloc[0], (list, set)):
                    _col = []
                    for val in df[col]:
                        if val is not None and len(val) > limit_n_entries:
                            ending = "]" if isinstance(val, list) else "}"
                            str_col = str(val[:limit_n_entries])[:-1] + ", ..." + ending
                        else:
                            str_col = str(val)

                        # escape } and { characters
                        str_col = str_col.replace("{", "\\{").replace("}", "\\}")
                        _col.append(str_col)
                    df[col] = _col

        if group_indices:
            df = df.set_index(group_indices)

        return df.to_latex()


def get_tasks(
    languages: list[str] | None = None,
    script: list[str] | None = None,
    domains: list[TASK_DOMAIN] | None = None,
    task_types: list[TASK_TYPE] | None = None,
    categories: list[TASK_CATEGORY] | None = None,
    tasks: list[str] | None = None,
    exclude_superseded: bool = True,
    eval_splits: list[str] | None = None,
    exclusive_language_filter: bool = False,
) -> MTEBTasks:
    """Get a list of tasks based on the specified filters.

    Args:
        languages: A list of languages either specified as 3 letter languages codes (ISO 639-3, e.g. "eng") or as script languages codes e.g.
            "eng-Latn". For multilingual tasks this will also remove languages that are not in the specified list.
        script: A list of script codes (ISO 15924 codes). If None, all scripts are included. For multilingual tasks this will also remove scripts
            that are not in the specified list.
        domains: A list of task domains.
        task_types: A string specifying the type of task. If None, all tasks are included.
        categories: A list of task categories these include "s2s" (sentence to sentence), "s2p" (sentence to paragraph) and "p2p" (paragraph to
            paragraph).
        tasks: A list of task names to include. If None, all tasks which pass the filters are included.
        exclude_superseded: A boolean flag to exclude datasets which are superseded by another.
        eval_splits: A list of evaluation splits to include. If None, all splits are included.
        exclusive_language_filter: Some datasets contains more than one language e.g. for STS22 the subset "de-en" contain eng and deu. If
            exclusive_language_filter is set to False both of these will be kept, but if set to True only those that contains all the languages
            specified will be kept.

    Returns:
        A list of all initialized tasks objects which pass all of the filters (AND operation).

    Examples:
        >>> get_tasks(languages=["eng", "deu"], script=["Latn"], domains=["Legal"])
        >>> get_tasks(languages=["eng"], script=["Latn"], task_types=["Classification"])
        >>> get_tasks(languages=["eng"], script=["Latn"], task_types=["Clustering"], exclude_superseded=False)
        >>> get_tasks(languages=["eng"], tasks=["WikipediaRetrievalMultilingual"], eval_splits=["test"])
        >>> get_tasks(tasks=["STS22"], languages=["eng"], exclusive_language_filter=True) # don't include multilingual subsets containing English
    """
    if tasks:
        _tasks = [
            get_task(
                task,
                languages,
                script,
                eval_splits=eval_splits,
                exclusive_language_filter=exclusive_language_filter,
            )
            for task in tasks
        ]
        return MTEBTasks(_tasks)

    _tasks = [
        cls().filter_languages(languages, script).filter_eval_splits(eval_splits)
        for cls in create_task_list()
    ]

    if languages:
        _tasks = filter_tasks_by_languages(_tasks, languages)
    if script:
        _tasks = filter_tasks_by_script(_tasks, script)
    if domains:
        _tasks = filter_tasks_by_domains(_tasks, domains)
    if task_types:
        _tasks = filter_tasks_by_task_types(_tasks, task_types)
    if categories:
        logger.warning(
            "`s2p`, `p2p`, and `s2s` will be removed and replaced by `t2t` in v2.0.0."
        )
        _tasks = filter_task_by_categories(_tasks, categories)
    if exclude_superseded:
        _tasks = filter_superseded_datasets(_tasks)

    return MTEBTasks(_tasks)


def get_task(
    task_name: str,
    languages: list[str] | None = None,
    script: list[str] | None = None,
    eval_splits: list[str] | None = None,
    hf_subsets: list[str] | None = None,
    exclusive_language_filter: bool = False,
) -> AbsTask:
    """Get a task by name.

    Args:
        task_name: The name of the task to fetch.
        languages: A list of languages either specified as 3 letter languages codes (ISO 639-3, e.g. "eng") or as script languages codes e.g.
            "eng-Latn". For multilingual tasks this will also remove languages that are not in the specified list.
        script: A list of script codes (ISO 15924 codes). If None, all scripts are included. For multilingual tasks this will also remove scripts
        eval_splits: A list of evaluation splits to include. If None, all splits are included.
        hf_subsets: A list of Huggingface subsets to evaluate on.
        exclusive_language_filter: Some datasets contains more than one language e.g. for STS22 the subset "de-en" contain eng and deu. If
            exclusive_language_filter is set to False both of these will be kept, but if set to True only those that contains all the languages
            specified will be kept.

    Returns:
        An initialized task object.

    Examples:
        >>> get_task("BornholmBitextMining")
    """
    if task_name not in TASKS_REGISTRY:
        close_matches = difflib.get_close_matches(task_name, TASKS_REGISTRY.keys())
        if close_matches:
            suggestion = (
                f"KeyError: '{task_name}' not found. Did you mean: {close_matches[0]}?"
            )
        else:
            suggestion = (
                f"KeyError: '{task_name}' not found and no similar keys were found."
            )
        raise KeyError(suggestion)
    task = TASKS_REGISTRY[task_name]()
    if eval_splits:
        task.filter_eval_splits(eval_splits=eval_splits)
    return task.filter_languages(
        languages,
        script,
        hf_subsets=hf_subsets,
        exclusive_language_filter=exclusive_language_filter,
    )



from __future__ import annotations

from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskRetrieval import AbsTaskRetrieval


class TRECCOVIDPL(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="TRECCOVID-PL",
        description="TRECCOVID is an ad-hoc search challenge based on the COVID-19 dataset containing scientific articles related to the COVID-19 pandemic.",
        reference="https://ir.nist.gov/covidSubmit/index.html",
        dataset={
            "path": "clarin-knext/trec-covid-pl",
            "revision": "81bcb408f33366c2a20ac54adafad1ae7e877fdd",
            "trust_remote_code": True,
        },
        type="Retrieval",
        category="s2p",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["pol-Latn"],
        main_score="ndcg_at_10",
        date=(
            "2019-12-01",
            "2022-12-31",
        ),  # approximate date of covid pandemic start and end (best guess)
        domains=["Academic", "Medical", "Non-fiction", "Written"],
        task_subtypes=["Article retrieval"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        sample_creation="machine-translated",
        bibtex_citation="""@misc{wojtasik2024beirpl,
      title={BEIR-PL: Zero Shot Information Retrieval Benchmark for the Polish Language}, 
      author={Konrad Wojtasik and Vadim Shishkin and Kacper Wołowiec and Arkadiusz Janz and Maciej Piasecki},
      year={2024},
      eprint={2305.19840},
      archivePrefix={arXiv},
      primaryClass={cs.IR}
}""",
        adapted_from=["TRECCOVID"],
    )



from __future__ import annotations

from .eng.Core17InstructionRetrieval import *
from .eng.News21InstructionRetrieval import *
from .eng.Robust04InstructionRetrieval import *
from .multilingual.mFollowIR import *



from __future__ import annotations

from functools import partial

from mteb.model_meta import ModelMeta
from mteb.models.instruct_wrapper import instruct_wrapper
from mteb.models.nvidia_models import nvidia_training_datasets

stella_en_400M = ModelMeta(
    # https://huggingface.co/dunzhang/stella_en_400M_v5/discussions/21#671a6205ac1e2416090f2bf4
    loader=partial(  # type: ignore
        instruct_wrapper,
        model_name_or_path="dunzhang/stella_en_400M_v5",
        attn="cccc",
        pooling_method="lasttoken",
        mode="embedding",
        torch_dtype="auto",
    ),
    name="dunzhang/stella_en_400M_v5",
    languages=["eng_Latn"],
    open_weights=True,
    use_instructions=True,
    revision="1bb50bc7bb726810eac2140e62155b88b0df198f",
    release_date="2024-07-12",
    n_parameters=435_000_000,
    memory_usage_mb=1660,
    max_tokens=8192,
    embed_dim=4096,
    license="mit",
    similarity_fn_name="cosine",
    framework=["Sentence Transformers", "PyTorch", "GritLM"],
    reference="https://huggingface.co/dunzhang/stella_en_400M_v5",
    training_datasets=nvidia_training_datasets,  # also distilled from gte-qwen (but training data is unknown) #2164
    public_training_code="https://github.com/NovaSearch-Team/RAG-Retrieval/blob/c40f4638b705eb77d88305d2056901ed550f9f4b/rag_retrieval/train/embedding/README.md",
    public_training_data=None,
)

stella_en_1_5b = ModelMeta(
    loader=partial(  # type: ignore
        instruct_wrapper,
        model_name_or_path="dunzhang/stella_en_1.5B_v5",
        attn="cccc",
        pooling_method="lasttoken",
        mode="embedding",
        torch_dtype="auto",
    ),
    name="dunzhang/stella_en_1.5B_v5",
    languages=["eng_Latn"],
    open_weights=True,
    use_instructions=True,
    revision="d03be74b361d4eb24f42a2fe5bd2e29917df4604",
    release_date="2024-07-12",
    n_parameters=1_540_000_000,
    memory_usage_mb=5887,
    max_tokens=131072,
    embed_dim=8960,
    license="mit",
    similarity_fn_name="cosine",
    framework=["Sentence Transformers", "PyTorch", "GritLM"],
    reference="https://huggingface.co/dunzhang/stella_en_1.5B_v5",
    training_datasets=nvidia_training_datasets,  # also distilled from gte-qwen (but training data is unknown) #2164
    public_training_code="https://github.com/NovaSearch-Team/RAG-Retrieval/blob/c40f4638b705eb77d88305d2056901ed550f9f4b/rag_retrieval/train/embedding/README.md",
    public_training_data=None,
)

stella_large_zh_v3_1792d = ModelMeta(
    name="dunzhang/stella-large-zh-v3-1792d",
    languages=["zho_Hans"],
    open_weights=True,
    revision="d5d39eb8cd11c80a63df53314e59997074469f09",
    release_date="2024-02-17",
    n_parameters=None,
    memory_usage_mb=None,  # can't see on model card
    embed_dim=1792,
    license="not specified",
    max_tokens=512,
    reference="https://huggingface.co/dunzhang/stella-large-zh-v3-1792d",
    similarity_fn_name="cosine",
    framework=["Sentence Transformers", "PyTorch"],
    use_instructions=False,
    superseded_by="dunzhang/stella-mrl-large-zh-v3.5-1792d",
    adapted_from=None,
    public_training_code=None,
    public_training_data=None,
    training_datasets={
        # Not in MTEB:
        # - infgrad/dialogue_rewrite_llm
        # - infgrad/retrieval_data_llm
    },
)

stella_base_zh_v3_1792d = ModelMeta(
    name="infgrad/stella-base-zh-v3-1792d",
    languages=["zho_Hans"],
    open_weights=True,
    revision="82254892a0fba125aa2abf3a4800d2dd12821343",
    release_date="2024-02-17",
    n_parameters=None,
    memory_usage_mb=None,  # can't see on model card
    embed_dim=1792,
    license="mit",
    max_tokens=512,
    reference="https://huggingface.co/infgrad/stella-base-zh-v3-1792d",
    similarity_fn_name="cosine",
    framework=["Sentence Transformers", "PyTorch"],
    use_instructions=False,
    superseded_by=None,
    adapted_from=None,
    public_training_code=None,
    public_training_data=None,
    training_datasets={
        # Not in MTEB:
        # - infgrad/dialogue_rewrite_llm
        # - infgrad/retrieval_data_llm
    },
)


stella_mrl_large_zh_v3_5_1792d = ModelMeta(
    name="dunzhang/stella-mrl-large-zh-v3.5-1792d",
    languages=["zho_Hans"],
    open_weights=True,
    revision="17bb1c32a93a8fc5f6fc9e91d5ea86da99983cfe",
    release_date="2024-02-27",
    n_parameters=int(326 * 1e6),
    memory_usage_mb=1242,
    embed_dim=1792,
    license="mit",
    max_tokens=512,
    reference="https://huggingface.co/dunzhang/stella-large-zh-v3-1792d",
    similarity_fn_name="cosine",
    framework=["Sentence Transformers", "PyTorch"],
    use_instructions=False,
    superseded_by=None,
    adapted_from="dunzhang/stella-large-zh-v3-1792d",
    public_training_code=None,
    public_training_data=None,
    training_datasets=None,  # Not specified
)

zpoint_large_embedding_zh = ModelMeta(
    name="iampanda/zpoint_large_embedding_zh",
    languages=["zho_Hans"],
    open_weights=True,
    revision="b1075144f440ab4409c05622c1179130ebd57d03",
    release_date="2024-06-04",
    n_parameters=int(326 * 1e6),
    memory_usage_mb=1242,
    embed_dim=1792,
    license="mit",
    max_tokens=512,
    reference="https://huggingface.co/iampanda/zpoint_large_embedding_zh",
    similarity_fn_name="cosine",
    framework=["Sentence Transformers", "PyTorch"],
    use_instructions=False,
    superseded_by=None,
    adapted_from="dunzhang/stella-mrl-large-zh-v3.5-1792d",
    public_training_code=None,
    public_training_data=None,
    training_datasets={
        # It's a bit unclear what they have trained on to be honest, because they don't list all
        # And they also have some rather cryptic description of their training procedure, but at
        # Least they disclose that they have trained on these:
        "MIRACLRetrieval": ["train"],
        "MIRACLReranking": ["train"],
        "DuRetrieval": ["train"],
        "T2Retrieval": ["train"],
        "MultiLongDocRetrieval": ["train"],
        #  Not in MTEB:
        #  - Shitao/bge-reranker-data
        #  - FreedomIntelligence/Huatuo26M-Lite
    },
)



from __future__ import annotations

from mteb.abstasks.Image.AbsTaskZeroshotClassification import (
    AbsTaskZeroshotClassification,
)
from mteb.abstasks.TaskMetadata import TaskMetadata


class SUN397ZeroshotClassification(AbsTaskZeroshotClassification):
    metadata = TaskMetadata(
        name="SUN397ZeroShot",
        description="Large scale scene recognition in 397 categories.",
        reference="https://ieeexplore.ieee.org/abstract/document/5539970",
        dataset={
            "path": "dpdl-benchmark/sun397",
            "revision": "7e6af6a2499ad708618be868e1471eac0aca1168",
        },
        type="ZeroShotClassification",
        category="i2t",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=(
            "2017-01-01",
            "2017-03-01",
        ),  # Estimated range for the collection of reviews
        domains=["Encyclopaedic"],
        task_subtypes=["Scene recognition"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        modalities=["image", "text"],
        sample_creation="created",
        bibtex_citation="""@INPROCEEDINGS{5539970,
        author={Xiao, Jianxiong and Hays, James and Ehinger, Krista A. and Oliva, Aude and Torralba, Antonio},
        booktitle={2010 IEEE Computer Society Conference on Computer Vision and Pattern Recognition}, 
        title={SUN database: Large-scale scene recognition from abbey to zoo}, 
        year={2010},
        volume={},
        number={},
        pages={3485-3492},
        doi={10.1109/CVPR.2010.5539970}}
        """,
        descriptive_stats={
            "n_samples": {"test": 21750},
            "avg_character_length": {"test": 256},
        },
    )

    def get_candidate_labels(self) -> list[str]:
        """Convert labels as such:
        - /b/boat_deck -> boat deck
        - /c/church/outdoor -> church outdoor
        """
        labels = []
        for name in self.dataset["test"].features[self.label_column_name].names:
            name = " ".join(name.split("/")[2:]).replace("_", " ")
            labels.append(f"a photo of a {name}.")
        return labels



from __future__ import annotations

import json
import logging
import os
from collections import defaultdict
from pathlib import Path
from time import asctime, time
from typing import Any

import tqdm
from datasets import Features, Value, load_dataset
from PIL import Image

from ...evaluation.evaluators import Any2AnyRetrievalEvaluator
from ..AbsTask import AbsTask, ScoresDict
from ..TaskMetadata import DescriptiveStatistics

logger = logging.getLogger(__name__)


class HFDataLoader:
    def __init__(
        self,
        hf_repo: str | None = None,
        hf_repo_qrels: str | None = None,
        data_folder: str | None = None,
        prefix: str | None = None,
        corpus_file: str = "corpus.jsonl",
        query_file: str = "queries.jsonl",
        qrels_folder: str = "qrels",
        qrels_file: str = "",
        streaming: bool = False,
        keep_in_memory: bool = False,
    ):
        self.corpus = {}
        self.queries = {}
        self.qrels = {}
        self.hf_repo = hf_repo
        if hf_repo:
            # By default fetch qrels from same repo not a second repo with "-qrels" like in original
            self.hf_repo_qrels = hf_repo_qrels if hf_repo_qrels else hf_repo
        else:
            # data folder would contain these files:
            # (1) fiqa/corpus.jsonl  (format: jsonlines)
            # (2) fiqa/queries.jsonl (format: jsonlines)
            # (3) fiqa/qrels/test.tsv (format: tsv ("\t"))
            if prefix:
                query_file = prefix + "-" + query_file
                qrels_folder = prefix + "-" + qrels_folder

            self.corpus_file = (
                os.path.join(data_folder, corpus_file) if data_folder else corpus_file
            )
            self.query_file = (
                os.path.join(data_folder, query_file) if data_folder else query_file
            )
            self.qrels_folder = (
                os.path.join(data_folder, qrels_folder) if data_folder else None
            )
            self.qrels_file = qrels_file
        self.streaming = streaming
        self.keep_in_memory = keep_in_memory

    @staticmethod
    def check(fIn: str, ext: str):
        if not os.path.exists(fIn):
            raise ValueError(f"File {fIn} not present! Please provide accurate file.")

        if not fIn.endswith(ext):
            raise ValueError(f"File {fIn} must be present with extension {ext}")

    def load(
        self, split="test"
    ) -> tuple[
        dict[str, dict[str, str | Image.Image]],
        dict[str, dict[str, str | Image.Image]],
        dict[str, dict[str, int]],
    ]:
        if not self.hf_repo:
            self.qrels_file = os.path.join(self.qrels_folder, split + ".tsv")
            self.check(fIn=self.corpus_file, ext="jsonl")
            self.check(fIn=self.query_file, ext="jsonl")
            self.check(fIn=self.qrels_file, ext="tsv")

        if not len(self.corpus):
            logger.info("Loading Corpus...")
            self._load_corpus()
            logger.info("Loaded %d %s Documents.", len(self.corpus), split.upper())
            logger.info("Doc Example: %s", self.corpus[0])

        if not len(self.queries):
            logger.info("Loading Queries...")
            self._load_queries(split)

        self._load_qrels(split)
        # filter queries with no qrels
        qrels_dict = defaultdict(dict)
        logger.info(f"{asctime()} - Done load qrels, before map qrels_dict")

        df = self.qrels.to_pandas()
        query_ids = df["query-id"].to_numpy()
        corpus_ids = df["corpus-id"].to_numpy()
        scores = df["score"].to_numpy()

        for q, c, s in zip(query_ids, corpus_ids, scores):
            qrels_dict[q][c] = int(s)
        logger.info(f"{asctime()} - Done qrels_dict")
        self.qrels = qrels_dict
        self.queries = self.queries.filter(lambda x: x["id"] in self.qrels)
        logger.info("Loaded %d %s Queries.", len(self.queries), split.upper())
        logger.info("Query Example: %s", self.queries[0])

        return self.corpus, self.queries, self.qrels

    def load_corpus(self) -> dict[str, dict[str, str]]:
        if not self.hf_repo:
            self.check(fIn=self.corpus_file, ext="jsonl")

        if not len(self.corpus):
            logger.info("Loading Corpus...")
            self._load_corpus()
            logger.info("Loaded %d %s Documents.", len(self.corpus))
            logger.info("Doc Example: %s", self.corpus[0])

        return self.corpus

    def _load_corpus(self):
        if self.hf_repo:
            corpus_ds = load_dataset(
                self.hf_repo,
                "corpus",
                keep_in_memory=self.keep_in_memory,
                streaming=self.streaming,
            )["corpus"]
        else:
            corpus_ds = load_dataset(
                "json",
                data_files=self.corpus_file,
                streaming=self.streaming,
                keep_in_memory=self.keep_in_memory,
            )
        self.corpus = corpus_ds

    def _load_queries(self, split):
        if self.hf_repo:
            queries_ds = load_dataset(
                self.hf_repo,
                "query",
                keep_in_memory=self.keep_in_memory,
                streaming=self.streaming,
            )[split]
        else:
            queries_ds = load_dataset(
                "json",
                data_files=self.query_file,
                streaming=self.streaming,
                keep_in_memory=self.keep_in_memory,
            )
        self.queries = queries_ds

    def _load_qrels(self, split):
        if self.hf_repo:
            qrels_ds = load_dataset(
                self.hf_repo_qrels,
                "qrels",
                keep_in_memory=self.keep_in_memory,
                streaming=self.streaming,
            )[split]
        else:
            qrels_ds = load_dataset(
                "csv",
                data_files=self.qrels_file,
                delimiter="\t",
                keep_in_memory=self.keep_in_memory,
            )

        if "Q0" in qrels_ds.column_names:
            qrels_ds = qrels_ds.remove_columns("Q0")
        features = Features(
            {
                "query-id": Value("string"),
                "corpus-id": Value("string"),
                "score": Value("float"),
            }
        )
        # Some datasets may have extra columns, e.g. `difficulty` in qrels for FORB.
        qrels_ds = qrels_ds.select_columns(["query-id", "corpus-id", "score"]).cast(
            features
        )
        self.qrels = qrels_ds


class Any2AnyRetrievalDescriptiveStatistics(DescriptiveStatistics):
    """Descriptive statistics for Retrieval

    Attributes:
        num_samples: Number of queries and documents
        num_queries: number of queries in the dataset
        num_documents: Number of documents
        number_of_characters: Total number of text characters in the dataset

        For text only:
        min_document_length: Minimum length of documents
        average_document_length: Average length of documents
        max_document_length: Maximum length of documents
        unique_documents: Number of unique documents

        For text only:
        min_query_length: Minimum length of queries
        average_query_length: Average length of queries
        max_query_length: Maximum length of queries
        unique_queries: Number of unique queries

        For images:
        num_query_images: Number of query images
        num_document_images: Number of document images

        min_relevant_docs_per_query: Minimum number of relevant documents per query
        average_relevant_docs_per_query: Average number of relevant documents per query
        max_relevant_docs_per_query: Maximum number of relevant documents per query
        unique_relevant_docs: Number of unique relevant documents
    """

    num_samples: int
    num_queries: int
    num_documents: int
    number_of_characters: int

    min_document_length: int
    average_document_length: float
    max_document_length: int
    unique_documents: int
    num_document_images: int

    min_query_length: int
    average_query_length: float
    max_query_length: int
    unique_queries: int
    num_query_images: int

    min_relevant_docs_per_query: int
    average_relevant_docs_per_query: float
    max_relevant_docs_per_query: int
    unique_relevant_docs: int


class AbsTaskAny2AnyRetrieval(AbsTask):
    """Abstract class for retrieval experiments.

    Child-classes must implement the following properties:

    self.corpus: dict[str, dict[str, str]]
        Semantically, it should contain dict[split_name, dict[sample_id, dict[str, str]]]
        E.g. {"test": {"document_one": {"_id": "d1", "title": "title", "text": "text"}}}

    self.queries: dict[str, dict[str, Union[str, List[str]]]]
        Semantically, it should contain dict[split_name, dict[sample_id, str]] or dict[split_name, dict[sample_id, List[str]]] for conversations
        E.g. {"test": {"q1": "query"}}
        or {"test": {"q1": ["turn1", "turn2", "turn3"]}}

    self.relevant_docs: dict[str, dict[str, dict[str, int]]]
        Semantically, it should contain dict[split_name, dict[sample_id, dict[doc_id, score]]]
        E.g.: {"test": {"q1": {"document_one": 1}}}
    """

    ignore_identical_ids: bool = False
    skip_first_result: bool = False

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def load_data(self, **kwargs):
        if self.data_loaded:
            return
        self.corpus, self.queries, self.relevant_docs = {}, {}, {}
        dataset_path = self.metadata_dict["dataset"]["path"]

        for split in kwargs.get("eval_splits", self.metadata_dict["eval_splits"]):
            corpus, queries, qrels = HFDataLoader(
                hf_repo=dataset_path,
                streaming=False,
                keep_in_memory=False,
            ).load(split=split)
            # directly pass in corpus and queries datasets now to prevent loading into memory
            # queries = {query["id"]: query for query in queries}
            # corpus = {doc["id"]: doc for doc in corpus}
            self.corpus[split], self.queries[split], self.relevant_docs[split] = (
                corpus,
                queries,
                qrels,
            )

        self.data_loaded = True

    def evaluate(
        self,
        model,
        split: str = "test",
        *,
        encode_kwargs: dict[str, Any] = {},
        **kwargs,
    ):
        retriever = Any2AnyRetrievalEvaluator(
            retriever=model,
            task_name=self.metadata.name,
            encode_kwargs=encode_kwargs,
            **kwargs,
        )

        scores = {}
        hf_subsets = list(self.hf_subsets) if self.is_multilingual else ["default"]

        for hf_subset in hf_subsets:
            logger.info(f"Subset: {hf_subset}")

            if hf_subset == "default":
                corpus, queries, relevant_docs = (
                    self.corpus[split],
                    self.queries[split],
                    self.relevant_docs[split],
                )
            else:
                corpus, queries, relevant_docs = (
                    self.corpus[hf_subset][split],
                    self.queries[hf_subset][split],
                    self.relevant_docs[hf_subset][split],
                )
            scores[hf_subset] = self._evaluate_subset(
                retriever, corpus, queries, relevant_docs, hf_subset, **kwargs
            )
        return scores

    def _evaluate_subset(
        self, retriever, corpus, queries, relevant_docs, hf_subset: str, **kwargs
    ):
        start_time = time()
        results = retriever(corpus, queries)
        end_time = time()
        logger.info(f"Time taken to retrieve: {end_time - start_time:.2f} seconds")

        save_predictions = kwargs.get("save_predictions", False)
        export_errors = kwargs.get("export_errors", False)
        if save_predictions or export_errors:
            output_folder = Path(kwargs.get("output_folder", "results"))
            if not os.path.isdir(output_folder):
                os.makedirs(output_folder)

        if save_predictions:
            top_k = kwargs.get("top_k", None)
            if top_k is not None:
                for qid in list(results.keys()):
                    doc_ids = set(
                        sorted(
                            results[qid], key=lambda x: results[qid][x], reverse=True
                        )[:top_k]
                    )
                    results[qid] = {
                        k: v for k, v in results[qid].items() if k in doc_ids
                    }
            qrels_save_path = (
                output_folder / f"{self.metadata.name}_{hf_subset}_predictions.json"
            )

            with open(qrels_save_path, "w") as f:
                json.dump(results, f)

        ndcg, _map, recall, precision, cv_recall, naucs = retriever.evaluate(
            relevant_docs,
            results,
            retriever.k_values,
            ignore_identical_ids=self.ignore_identical_ids,
            skip_first_result=self.skip_first_result,
        )
        mrr, naucs_mrr = retriever.evaluate_custom(
            relevant_docs, results, retriever.k_values, "mrr"
        )
        scores = {
            **{f"ndcg_at_{k.split('@')[1]}": v for (k, v) in ndcg.items()},
            **{f"map_at_{k.split('@')[1]}": v for (k, v) in _map.items()},
            **{f"recall_at_{k.split('@')[1]}": v for (k, v) in recall.items()},
            **{f"cv_recall_at_{k.split('@')[1]}": v for (k, v) in cv_recall.items()},
            **{f"precision_at_{k.split('@')[1]}": v for (k, v) in precision.items()},
            **{f"mrr_at_{k.split('@')[1]}": v for (k, v) in mrr.items()},
            **{
                k.replace("@", "_at_").replace("_P", "_precision").lower(): v
                for k, v in naucs.items()
            },
            **{
                k.replace("@", "_at_").replace("_P", "_precision").lower(): v
                for k, v in naucs_mrr.items()
            },
        }
        self._add_main_score(scores)

        if export_errors:
            errors = {}

            top_k = kwargs.get("top_k", 1)
            if not save_predictions and top_k == 1:
                for qid in results.keys():
                    doc_scores = results[qid]
                    sorted_docs = sorted(
                        doc_scores.items(), key=lambda x: x[1], reverse=True
                    )[:top_k]
                    results[qid] = dict(sorted_docs)
            for qid, retrieved_docs in results.items():
                expected_docs = relevant_docs[qid]
                false_positives = [
                    doc for doc in retrieved_docs if doc not in expected_docs
                ]
                false_negatives = [
                    doc for doc in expected_docs if doc not in retrieved_docs
                ]
                if false_positives or false_negatives:
                    errors[qid] = {
                        "false_positives": false_positives,
                        "false_negatives": false_negatives,
                    }

            errors_save_path = (
                output_folder / f"{self.metadata.name}_{hf_subset}_errors.json"
            )
            with open(errors_save_path, "w") as f:
                json.dump(errors, f)

        return scores

    def _add_main_score(self, scores: ScoresDict) -> None:
        scores["main_score"] = scores[self.metadata.main_score]

    def _calculate_metrics_from_split(
        self, split: str, hf_subset: str | None = None, compute_overall: bool = False
    ) -> Any2AnyRetrievalDescriptiveStatistics:
        if hf_subset:
            queries = self.queries[hf_subset][split]
            corpus = self.corpus[hf_subset][split]
            relevant_docs = self.relevant_docs[hf_subset][split]
        elif compute_overall:
            queries = {}
            corpus = {}
            relevant_docs = {}
            for hf_subset in self.metadata.eval_langs:
                queries.update(process_docs(self.queries, hf_subset, split))
                corpus.update(process_docs(self.corpus, hf_subset, split))
                relevant_docs.update(
                    process_relevant_docs(self.relevant_docs, hf_subset, split)
                )
        else:
            queries = self.queries[split]
            corpus = self.corpus[split]
            relevant_docs = self.relevant_docs[split]

        queries_lens, doc_lens = [], []
        num_query_images = 0
        num_document_images = 0

        q_modality = queries[0]["modality"]
        unique_queries = len(set(queries["text"])) if "text" in q_modality else 0

        for query in tqdm.tqdm(queries, desc="queries:"):
            if "text" in q_modality:
                text_query = query["text"]
                queries_lens.append(len(text_query))
            if "image" in q_modality:
                num_query_images += 1

        d_modality = corpus[0]["modality"]
        unique_documents = len(set(corpus["text"])) if "text" in d_modality else 0

        for doc in tqdm.tqdm(corpus, desc="docs:"):
            if "text" in d_modality:
                text_doc = doc["text"]
                doc_lens.append(len(text_doc))
            if "image" in d_modality:
                num_document_images += 1

        total_doc_len = sum(doc_lens)
        total_query_len = sum(queries_lens)
        num_documents = len(corpus)
        num_queries = len(queries)

        # create a list of number of relevant docs per query
        queries_set = set(queries["id"])
        qrels_lengths = [
            len(relevant_docs[qid])
            for qid in tqdm.tqdm(relevant_docs.keys(), desc="qrels:")
            if qid in queries_set
        ]
        num_qrels = sum(qrels_lengths)
        qrels_per_doc = num_qrels / len(relevant_docs) if num_queries else 0
        unique_qrels = len({doc for qid in relevant_docs for doc in relevant_docs[qid]})

        return Any2AnyRetrievalDescriptiveStatistics(
            number_of_characters=total_query_len + total_doc_len,
            num_samples=num_documents + num_queries,
            num_queries=num_queries,
            num_documents=num_documents,
            min_document_length=min(doc_lens) if doc_lens else 0,
            average_document_length=total_doc_len / len(doc_lens) if doc_lens else 0,
            max_document_length=max(doc_lens) if doc_lens else 0,
            unique_documents=unique_documents,
            num_document_images=num_document_images,
            min_query_length=min(queries_lens) if queries_lens else 0,
            average_query_length=total_query_len / len(queries_lens)
            if queries_lens
            else 0,
            max_query_length=max(queries_lens) if queries_lens else 0,
            unique_queries=unique_queries,
            num_query_images=num_query_images,
            min_relevant_docs_per_query=min(qrels_lengths),
            average_relevant_docs_per_query=qrels_per_doc,
            max_relevant_docs_per_query=max(qrels_lengths),
            unique_relevant_docs=unique_qrels,
        )


def process_relevant_docs(
    collection: dict[str, dict[str, dict[str, dict[str, int]]]],
    hf_subset: str,
    split: str,
) -> dict[str, dict[str, int]]:
    """Collections can contain overlapping ids in different splits. Prepend split to avoid this"""
    return_collection = {}
    for query_id, relevant in collection[hf_subset][split].items():
        return_collection[f"{split}_{hf_subset}_{query_id}"] = {
            f"{split}_{hf_subset}_{doc_id}": value for doc_id, value in relevant.items()
        }
    return return_collection


def process_docs(
    collection: dict[str, dict[str, dict[str, str] | str]], hf_subset: str, split: str
) -> dict[str, str]:
    """Collections can contain overlapping ids in different splits. Prepend split to avoid this"""
    return {
        f"{split}_{hf_subset}_{k}": v for k, v in collection[hf_subset][split].items()
    }



"""Mock models to be used for testing"""

from __future__ import annotations

from collections.abc import Sequence
from typing import Any, Literal

import numpy as np
import torch
from numpy import ndarray
from sentence_transformers import CrossEncoder, SentenceTransformer
from torch import Tensor
from torch.utils.data import DataLoader

import mteb
from mteb import SentenceTransformerWrapper
from mteb.encoder_interface import PromptType
from mteb.model_meta import ModelMeta
from tests.test_benchmark.task_grid import MOCK_TASK_TEST_GRID


class MockNumpyEncoder(mteb.Encoder):
    def __init__(self):
        pass

    def encode(self, sentences, prompt_name: str | None = None, **kwargs):
        return np.random.rand(len(sentences), 10)


class MockTorchEncoder(mteb.Encoder):
    def __init__(self):
        pass

    def encode(self, sentences, prompt_name: str | None = None, **kwargs):
        return torch.randn(len(sentences), 10).numpy()


class MockTorchbf16Encoder(SentenceTransformer):
    def __init__(self):
        pass

    def encode(self, sentences, prompt_name: str | None = None, **kwargs):
        return torch.randn(len(sentences), 10, dtype=torch.bfloat16)


class MockCLIPEncoder:
    mteb_model_meta = ModelMeta(
        name="MockCLIPModel",
        languages=["eng_Latn"],
        revision="3d74acf9a28c67741b2f4f2ea7635f0aaf6f0268",
        release_date="2021-02-06",
        modalities=["image", "text"],
        n_parameters=86_600_000,
        memory_usage_mb=330,
        max_tokens=None,
        embed_dim=768,
        license=None,
        open_weights=True,
        public_training_code=None,
        public_training_data=None,
        framework=["PyTorch"],
        reference="https://huggingface.co/openai/clip-vit-base-patch32",
        similarity_fn_name=None,
        use_instructions=False,
        training_datasets=None,
    )
    model_card_data = mteb_model_meta

    def __init__(self):
        pass

    def get_text_embeddings(self, texts, **kwargs):
        return torch.randn(len(texts), 10)

    def get_image_embeddings(self, images, **kwargs):
        if isinstance(images, DataLoader):
            all_embeddings = []
            for batch in images:
                batch_embeddings = torch.randn(len(batch), 10)
                all_embeddings.append(batch_embeddings)
            return torch.cat(all_embeddings, dim=0)
        else:
            return torch.randn(len(images), 10)

    def get_fused_embeddings(self, texts, images, **kwargs):
        return torch.randn(len(texts), 10)

    def calculate_probs(self, text_embeddings, image_embeddings):
        return torch.randn(image_embeddings.shape[0], text_embeddings.shape[0])


class MockMocoEncoder:
    mteb_model_meta = ModelMeta(
        name="MockMocoModel",
        languages=["eng_Latn"],
        revision="7d091cd70772c5c0ecf7f00b5f12ca609a99d69d",
        release_date="2024-01-01",
        modalities=["image"],
        n_parameters=86_600_000,
        memory_usage_mb=330,
        max_tokens=None,
        embed_dim=768,
        license=None,
        open_weights=True,
        public_training_code=None,
        public_training_data=None,
        framework=["PyTorch"],
        reference="https://github.com/facebookresearch/moco-v3",
        similarity_fn_name=None,
        use_instructions=False,
        training_datasets=None,
    )
    model_card_data = mteb_model_meta

    def __init__(self):
        pass

    def get_text_embeddings(self, texts, **kwargs):
        pass

    def get_image_embeddings(self, images, **kwargs):
        pass

    def get_fused_embeddings(self, texts, images, **kwargs):
        pass

    def calculate_probs(self, text_embeddings, image_embeddings):
        pass


class MockSentenceTransformer(SentenceTransformer):
    """A mock implementation of the SentenceTransformer intended to implement just the encode, method using the same arguments."""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def encode(
        self,
        sentences: str | list[str],
        prompt_name: str | None = None,
        prompt: str | None = None,
        batch_size: int = 32,
        show_progress_bar: bool | None = None,
        output_value: Literal["sentence_embedding", "token_embeddings"]
        | None = "sentence_embedding",
        precision: Literal["float32", "int8", "uint8", "binary", "ubinary"] = "float32",
        convert_to_numpy: bool = True,
        convert_to_tensor: bool = False,
        device: str | None = None,
        normalize_embeddings: bool = False,
    ) -> list[Tensor] | ndarray | Tensor:
        return torch.randn(len(sentences), 10).numpy()


class MockSentenceTransformerWrapper(SentenceTransformerWrapper):
    def __init__(
        self,
        model: str | SentenceTransformer | CrossEncoder,
        revision: str | None = None,
        model_prompts: dict[str, str] | None = None,
        **kwargs,
    ) -> None:
        """Wrapper for SentenceTransformer models.

        Args:
            model: The SentenceTransformer model to use. Can be a string (model name), a SentenceTransformer model, or a CrossEncoder model.
            revision: The revision of the model to use.
            model_prompts: A dictionary mapping task names to prompt names.
                First priority is given to the composed prompt of task name + prompt type (query or passage), then to the specific task prompt,
                then to the composed prompt of task type + prompt type, then to the specific task type prompt,
                and finally to the specific prompt type.
            **kwargs: Additional arguments to pass to the SentenceTransformer model.
        """
        if isinstance(model, str):
            self.model = SentenceTransformer(
                model, revision=revision, trust_remote_code=True, **kwargs
            )
        else:
            self.model = model

        if (
            model_prompts is None
            and hasattr(self.model, "prompts")
            and len(self.model.prompts) > 0
        ):
            model_prompts = self.model.prompts
        elif model_prompts is not None and hasattr(self.model, "prompts"):
            self.model.prompts = model_prompts
        self.model_prompts = model_prompts

    def encode(
        self,
        sentences: Sequence[str],
        *,
        task_name: str,
        prompt_type: PromptType | None = None,
        **kwargs: Any,
    ) -> np.ndarray:
        prompt_name = None
        if self.model_prompts is not None:
            prompt_name = get_mock_prompt_name(
                self.model_prompts, task_name, prompt_type
            )

        embeddings = self.model.encode(
            sentences,
            prompt_name=prompt_name,
            **kwargs,  # sometimes in kwargs can be return_tensors=True
        )
        if isinstance(embeddings, torch.Tensor):
            embeddings = embeddings.cpu().detach().float().numpy()
        return embeddings


def get_mock_prompt_name(
    task_to_prompt: dict[str, str], task_name: str, prompt_type: PromptType | None
) -> str | None:
    task = [
        mock_task
        for mock_task in MOCK_TASK_TEST_GRID
        if mock_task.metadata.name == task_name
    ][0]
    task_type = task.metadata.type
    prompt_type_value = prompt_type.value if prompt_type else None

    if (
        task_name
        and prompt_type
        and f"{task_name}-{prompt_type_value}" in task_to_prompt
    ):
        return f"{task_name}-{prompt_type_value}"
    if task_name and task_name in task_to_prompt:
        return task_name
    if (
        task_type
        and prompt_type
        and f"{task_type}-{prompt_type_value}" in task_to_prompt
    ):
        return f"{task_type}-{prompt_type_value}"
    if task_type and task_type in task_to_prompt:
        return task_type
    if prompt_type and prompt_type in task_to_prompt:
        return prompt_type_value
    return None



from __future__ import annotations

from .aggregated_tasks import *
from .BitextMining import *
from .Classification import *
from .Clustering import *
from .Image.Any2AnyMultiChoice import *
from .Image.Any2AnyRetrieval import *
from .Image.Clustering import *
from .Image.ImageClassification import *
from .Image.ImageMultilabelClassification import *
from .Image.ImageTextPairClassification import *
from .Image.VisualSTS import *
from .Image.ZeroshotClassification import *
from .InstructionRetrieval import *
from .MultiLabelClassification import *
from .PairClassification import *
from .Reranking import *
from .Retrieval import *
from .SpeedTask import *
from .STS import *
from .Summarization import *



from __future__ import annotations

from mteb.abstasks.Image.AbsTaskAny2AnyRetrieval import AbsTaskAny2AnyRetrieval
from mteb.abstasks.TaskMetadata import TaskMetadata


class TUBerlinT2IRetrieval(AbsTaskAny2AnyRetrieval):
    metadata = TaskMetadata(
        name="TUBerlinT2IRetrieval",
        description="Retrieve sketch images based on text descriptions.",
        reference="https://dl.acm.org/doi/pdf/10.1145/2185520.2185540?casa_token=tq-eUx5UROYAAAAA:_694nPzE7tali6LCkxQc0M-mlo9xslasPMcVnFPMy9tDfvt7lg7p1RTe-k8VWCjuv9gmkQqasKUZ",
        dataset={
            "path": "gowitheflow/tu-berlin",
            "revision": "0cd78cd1ddbd3cafa9f319c638ebd77836ec9ff6",
            # "trust_remote_code": True,
        },
        type="Any2AnyRetrieval",
        category="t2i",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=("2012-01-01", "2012-12-31"),
        domains=["Encyclopaedic"],
        task_subtypes=["Image Text Retrieval"],
        license="cc-by-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        modalities=["text", "image"],
        sample_creation="found",
        bibtex_citation="""@article{eitz2012humans,
  title={How do humans sketch objects?},
  author={Eitz, Mathias and Hays, James and Alexa, Marc},
  journal={ACM Transactions on graphics (TOG)},
  volume={31},
  number={4},
  pages={1--10},
  year={2012},
  publisher={Acm New York, NY, USA}
}""",
        descriptive_stats={
            "n_samples": {"test": 250},
            "avg_character_length": {
                "test": {
                    "average_document_length": 0.0,
                    "average_query_length": 7.24,
                    "num_documents": 20000,
                    "num_queries": 250,
                    "average_relevant_docs_per_query": 80.0,
                }
            },
        },
    )



from __future__ import annotations

from mteb.abstasks.AbsTaskSTS import AbsTaskSTS
from mteb.abstasks.TaskMetadata import TaskMetadata


class SickrPLSTS(AbsTaskSTS):
    metadata = TaskMetadata(
        name="SICK-R-PL",
        dataset={
            "path": "PL-MTEB/sickr-pl-sts",
            "revision": "fd5c2441b7eeff8676768036142af4cfa42c1339",
        },
        description="Polish version of SICK dataset for textual relatedness.",
        reference="https://aclanthology.org/2020.lrec-1.207",
        type="STS",
        category="s2s",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["pol-Latn"],
        main_score="cosine_spearman",
        date=("2018-01-01", "2019-09-01"),  # rough estimate
        domains=["Web", "Written"],
        task_subtypes=["Textual Entailment"],
        license="cc-by-nc-sa-3.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="human-translated and localized",
        bibtex_citation="""
@inproceedings{dadas-etal-2020-evaluation,
    title = "Evaluation of Sentence Representations in {P}olish",
    author = "Dadas, Slawomir  and
      Perelkiewicz, Michal  and
      Poswiata, Rafal",
    editor = "Calzolari, Nicoletta  and
      B{\'e}chet, Fr{\'e}d{\'e}ric  and
      Blache, Philippe  and
      Choukri, Khalid  and
      Cieri, Christopher  and
      Declerck, Thierry  and
      Goggi, Sara  and
      Isahara, Hitoshi  and
      Maegaard, Bente  and
      Mariani, Joseph  and
      Mazo, Helene  and
      Moreno, Asuncion  and
      Odijk, Jan  and
      Piperidis, Stelios",
    booktitle = "Proceedings of the Twelfth Language Resources and Evaluation Conference",
    month = may,
    year = "2020",
    address = "Marseille, France",
    publisher = "European Language Resources Association",
    url = "https://aclanthology.org/2020.lrec-1.207",
    pages = "1674--1680",
    language = "English",
    ISBN = "979-10-95546-34-4",
}
        """,
    )

    @property
    def metadata_dict(self) -> dict[str, str]:
        metadata_dict = super().metadata_dict
        metadata_dict["min_score"] = 1
        metadata_dict["max_score"] = 5
        return metadata_dict


class CdscrSTS(AbsTaskSTS):
    metadata = TaskMetadata(
        name="CDSC-R",
        dataset={
            "path": "PL-MTEB/cdscr-sts",
            "revision": "1cd6abbb00df7d14be3dbd76a7dcc64b3a79a7cd",
        },
        description="Compositional Distributional Semantics Corpus for textual relatedness.",
        reference="https://aclanthology.org/P17-1073.pdf",
        type="STS",
        category="s2s",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["pol-Latn"],
        main_score="cosine_spearman",
        date=("2016-01-01", "2017-04-01"),  # rough estimate
        domains=["Web", "Written"],
        task_subtypes=["Textual Entailment"],
        license="cc-by-nc-sa-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="human-translated and localized",
        bibtex_citation="""
@inproceedings{wroblewska-krasnowska-kieras-2017-polish,
    title = "{P}olish evaluation dataset for compositional distributional semantics models",
    author = "Wr{\'o}blewska, Alina  and
      Krasnowska-Kiera{\'s}, Katarzyna",
    editor = "Barzilay, Regina  and
      Kan, Min-Yen",
    booktitle = "Proceedings of the 55th Annual Meeting of the Association for Computational Linguistics (Volume 1: Long Papers)",
    month = jul,
    year = "2017",
    address = "Vancouver, Canada",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/P17-1073",
    doi = "10.18653/v1/P17-1073",
    pages = "784--792",
}

        """,
    )

    @property
    def metadata_dict(self) -> dict[str, str]:
        metadata_dict = super().metadata_dict
        metadata_dict["min_score"] = 1
        metadata_dict["max_score"] = 5
        return metadata_dict



from __future__ import annotations

from mteb.abstasks.Image.AbsTaskAny2AnyRetrieval import AbsTaskAny2AnyRetrieval
from mteb.abstasks.TaskMetadata import TaskMetadata


class FashionIQIT2IRetrieval(AbsTaskAny2AnyRetrieval):
    metadata = TaskMetadata(
        name="FashionIQIT2IRetrieval",
        description="Retrieve clothes based on descriptions.",
        reference="https://openaccess.thecvf.com/content/CVPR2021/html/Wu_Fashion_IQ_A_New_Dataset_Towards_Retrieving_Images_by_Natural_CVPR_2021_paper.html",
        dataset={
            "path": "MRBench/mbeir_fashioniq_task7",
            "revision": "e6f0ec70becc413d940cd62b2cfa3b1d3a08c31a",
            # "trust_remote_code": True,
        },
        type="Any2AnyRetrieval",
        category="it2i",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=("2021-01-01", "2021-12-31"),
        domains=["Encyclopaedic"],
        task_subtypes=["Image Text Retrieval"],
        license="apache-2.0",
        annotations_creators="derived",
        dialect=[],
        modalities=["text", "image"],
        sample_creation="created",
        bibtex_citation="""@inproceedings{wu2021fashion,
  title={Fashion iq: A new dataset towards retrieving images by natural language feedback},
  author={Wu, Hui and Gao, Yupeng and Guo, Xiaoxiao and Al-Halah, Ziad and Rennie, Steven and Grauman, Kristen and Feris, Rogerio},
  booktitle={Proceedings of the IEEE/CVF Conference on computer vision and pattern recognition},
  pages={11307--11317},
  year={2021}
}""",
        prompt={
            "query": "Find a fashion image that aligns with the reference image and style note."
        },
        descriptive_stats={
            "n_samples": {"test": 6003},
            "avg_character_length": {
                "test": {
                    "average_document_length": 0.0,
                    "average_query_length": 0.0,
                    "num_documents": 74381,
                    "num_queries": 6003,
                    "average_relevant_docs_per_query": 1.0,
                }
            },
        },
    )



from __future__ import annotations

from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskRetrieval import AbsTaskRetrieval


class RiaNewsRetrieval(AbsTaskRetrieval):
    ignore_identical_ids = True

    metadata = TaskMetadata(
        name="RiaNewsRetrieval",
        dataset={
            "path": "ai-forever/ria-news-retrieval",
            "revision": "82374b0bbacda6114f39ff9c5b925fa1512ca5d7",
        },
        description="News article retrieval by headline. Based on Rossiya Segodnya dataset.",
        reference="https://arxiv.org/abs/1901.07786",
        type="Retrieval",
        category="s2p",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["rus-Cyrl"],
        main_score="ndcg_at_10",
        date=("2010-01-01", "2014-12-31"),
        domains=["News", "Written"],
        task_subtypes=["Article retrieval"],
        license="cc-by-nc-nd-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation="""@inproceedings{gavrilov2018self,
        title={Self-Attentive Model for Headline Generation},
        author={Gavrilov, Daniil and  Kalaidin, Pavel and  Malykh, Valentin},
        booktitle={Proceedings of the 41st European Conference on Information Retrieval},
        year={2019}
        }""",
        prompt={"query": "Given a news title, retrieve relevant news article"},
    )


class RiaNewsRetrievalHardNegatives(AbsTaskRetrieval):
    ignore_identical_ids = True

    metadata = TaskMetadata(
        name="RiaNewsRetrievalHardNegatives",
        dataset={
            "path": "mteb/RiaNewsRetrieval_test_top_250_only_w_correct-v2",
            "revision": "d42860a6c15f0a2c4485bda10c6e5b641fdfe479",
        },
        description="News article retrieval by headline. Based on Rossiya Segodnya dataset. The hard negative version has been created by pooling the 250 top documents per query from BM25, e5-multilingual-large and e5-mistral-instruct.",
        reference="https://arxiv.org/abs/1901.07786",
        type="Retrieval",
        category="s2p",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["rus-Cyrl"],
        main_score="ndcg_at_10",
        date=("2010-01-01", "2014-12-31"),
        domains=["News", "Written"],
        task_subtypes=["Article retrieval"],
        license="cc-by-nc-nd-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation="""@inproceedings{gavrilov2018self,
        title={Self-Attentive Model for Headline Generation},
        author={Gavrilov, Daniil and  Kalaidin, Pavel and  Malykh, Valentin},
        booktitle={Proceedings of the 41st European Conference on Information Retrieval},
        year={2019}
        }""",
        adapted_from=["RiaNewsRetrieval"],
    )



from __future__ import annotations

from functools import partial

import torch

from mteb.model_meta import ModelMeta, sentence_transformers_loader
from mteb.models.bge_models import bge_m3_training_data, bge_training_data
from mteb.models.e5_instruct import E5_MISTRAL_TRAINING_DATA
from mteb.models.e5_models import E5_TRAINING_DATA
from mteb.models.sentence_transformers_models import sent_trf_training_dataset

Haon_Chen__speed_embedding_7b_instruct = ModelMeta(
    name="Haon-Chen/speed-embedding-7b-instruct",
    revision="c167e9a8144b397622ce47b85d9edcdeecef3d3f",
    release_date="2024-10-31",
    languages=["eng_Latn"],
    loader=None,
    n_parameters=7110660096,
    memory_usage_mb=13563,
    max_tokens=32768.0,
    embed_dim=None,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch"],
    reference="https://huggingface.co/Haon-Chen/speed-embedding-7b-instruct",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=None,
    adapted_from="mistralai/Mistral-7B-v0.1",
    superseded_by=None,
)
Gameselo__STS_multilingual_mpnet_base_v2 = ModelMeta(
    name="Gameselo/STS-multilingual-mpnet-base-v2",
    revision="449f917af30f590fc31f9ffb226c94f21a2f47b8",
    release_date="2024-06-07",
    languages=[],
    loader=None,
    n_parameters=278043648,
    memory_usage_mb=1061,
    max_tokens=514.0,
    embed_dim=768,
    license=None,
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/Gameselo/STS-multilingual-mpnet-base-v2",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=None,
    adapted_from="sentence-transformers/paraphrase-multilingual-mpnet-base-v2",
    superseded_by=None,
)

kalm_training_data = {
    # from technical report
    # not in MTEB:
    # ExpertQA
    # MEDI2BGE
    # OpenOrca
    # PAQ
    # PubMedQA
    # SearchQA
    # arxiv_qa
    # rag-dataset-12000
    # CC-News
    # SQuAD 2.0
    # TriviaQA
    # WebGPT Comparisons
    # MultiNLI
    # NLLB
    # WikiAnswers
    # SimCSE NLI
    # SNLI
    # Aya Dataset
    # eli5
    # ----
    # in MTEB:
    "CodeFeedbackMT": ["train"],
    "CodeFeedbackST": ["train"],
    "ArxivClusteringP2P": ["train"],
    "ArxivClusteringS2S": ["train"],
    "ArxivClusteringP2P.v2": ["train"],
    "TRECCOVID": ["train"],
    "DBPedia": ["train"],
    "ESCIReranking": ["train"],
    "FEVER": ["train"],
    "FiQA2018": ["train"],
    "FEVERHardNegatives": ["train"],
    "NanoFEVERRetrieval": ["train"],
    "FEVER-NL": ["train"],  # translation not trained on
    "FiQA2018-NL": ["train"],  # translation not trained on
    "HotpotQA-PL": ["train"],  # translation not trained on
    "HotpotQA-NL": ["train"],  # translation not trained on
    "HotpotQAHardNegatives": ["train"],
    "MultiLongDocRetrieval": ["train"],
    "MSMARCO": ["train"],
    "MSMARCOHardNegatives": ["train"],
    "NanoMSMARCORetrieval": ["train"],
    "MSMARCO-PL": ["train"],  # translation not trained on
    "mMARCO-NL": ["train"],  # translation not trained on
    "MSMARCOv2": ["train"],
    "NFCorpus": ["train"],
    "SciFact": ["train"],
    "NQ": ["train"],
    "NQHardNegatives": ["train"],
    "NanoNQRetrieval": ["train"],
    "NQ-PL": ["train"],  # translation not trained on
    "NQ-NL": ["train"],  # translation not trained on
    "YahooAnswersTopicsClassification": ["train"],
    "ContractNLIConfidentialityOfAgreementLegalBenchClassification": ["train"],
    "ContractNLIExplicitIdentificationLegalBenchClassification": ["train"],
    "ContractNLIInclusionOfVerballyConveyedInformationLegalBenchClassification": [
        "train"
    ],
    "ContractNLILimitedUseLegalBenchClassification": ["train"],
    "ContractNLINoLicensingLegalBenchClassification": ["train"],
    "ContractNLINoticeOnCompelledDisclosureLegalBenchClassification": ["train"],
    "ContractNLIPermissibleAcquirementOfSimilarInformationLegalBenchClassification": [
        "train"
    ],
    "ContractNLIPermissibleCopyLegalBenchClassification": ["train"],
    "ContractNLIPermissibleDevelopmentOfSimilarInformationLegalBenchClassification": [
        "train"
    ],
    "ContractNLIPermissiblePostAgreementPossessionLegalBenchClassification": ["train"],
    "ContractNLIReturnOfConfidentialInformationLegalBenchClassification": ["train"],
    "ContractNLISharingWithEmployeesLegalBenchClassification": ["train"],
    "ContractNLISharingWithThirdPartiesLegalBenchClassification": ["train"],
    "ContractNLISurvivalOfObligationsLegalBenchClassification": ["train"],
    "QuoraRetrieval": ["train"],
    "NanoQuoraRetrieval": ["train"],
    "BiorxivClusteringP2P.v2": ["train"],
    "BiorxivClusteringS2S.v2": ["train"],
    "MedrxivClusteringP2P.v2": ["train"],
    "MedrxivClusteringS2S.v2": ["train"],
    "Banking77Classification": ["train"],
    "AmazonPolarityClassification": ["train"],
    "ImdbClassification": ["train"],
    "EmotionClassification": ["train"],
    "TweetSentimentExtractionClassification": ["train"],
    "ToxicConversationsClassification": ["train"],
    "MIRACLRetrieval": ["train"],
    "MIRACLRetrievalHardNegatives": ["train"],
    "MIRACLReranking": ["train"],
    "MrTidyRetrieval": ["train"],
    "PawsXPairClassification": ["train"],
    "AmazonReviewsClassification": ["train"],
    "AmazonCounterfactualClassification": ["train"],
    "MultilingualSentiment": ["train"],
    "MassiveIntentClassification": ["train"],
    "MassiveScenarioClassification": ["train"],
    "MTOPDomainClassification": ["train"],
    "MTOPIntentClassification": ["train"],
}

HIT_TMG__KaLM_embedding_multilingual_mini_instruct_v1 = ModelMeta(
    name="HIT-TMG/KaLM-embedding-multilingual-mini-instruct-v1",
    revision="45e42c89990c40aca042659133fc8b13c28634b5",
    release_date="2024-10-23",
    languages=None,
    loader=None,
    n_parameters=494032768,
    memory_usage_mb=1885,
    max_tokens=512,
    embed_dim=896,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch"],
    reference="https://huggingface.co/HIT-TMG/KaLM-embedding-multilingual-mini-instruct-v1",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=kalm_training_data,
    adapted_from="/mnt/shgeminicephfs/wx-dc-plt-hpc/xinshuohu/Output/Embedding/Qwen2-0.5B-eos_mean_pretrain_0806_1e-4_uen_sft_1022_filtered_v2_inst_3node_g8_1e-5_sin-0.1_mrl",
    superseded_by=None,
)
HIT_TMG__KaLM_embedding_multilingual_mini_v1 = ModelMeta(
    name="HIT-TMG/KaLM-embedding-multilingual-mini-v1",
    revision="8a82a0cd2b322b91723e252486f7cce6fd8ac9d3",
    release_date="2024-08-27",
    languages=None,
    loader=None,
    n_parameters=494032768,
    memory_usage_mb=1885,
    max_tokens=512,
    embed_dim=896,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch"],
    reference="https://huggingface.co/HIT-TMG/KaLM-embedding-multilingual-mini-v1",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=kalm_training_data,
    adapted_from="/mnt/shgeminicephfs/wx-dc-plt-hpc/xinshuohu/Output/Embedding/Qwen2-0.5B-eos_mean_pretrain_0806_1e-4_uen_sft_0902_filtered_v2_3node_g8_1e-5_sin-0.1",
    superseded_by=None,
)
Hum_Works__lodestone_base_4096_v1 = ModelMeta(
    name="Hum-Works/lodestone-base-4096-v1",
    revision="9bbc2d0b57dd2198aea029404b0f976712a7d966",
    release_date="2023-08-25",
    languages=["eng_Latn"],
    loader=None,
    n_parameters=None,
    memory_usage_mb=None,
    max_tokens=None,
    embed_dim=768,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch"],
    reference="https://huggingface.co/Hum-Works/lodestone-base-4096-v1",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets={
        "MSMARCO": ["train"],
        "MSMARCOHardNegatives": ["train"],
        "NanoMSMARCORetrieval": ["train"],
        "MSMARCO-PL": ["train"],  # translation not trained on
        "mMARCO-NL": ["train"],  # translation not trained on
        "NQ": ["train"],
        "NQHardNegatives": ["train"],
        "NanoNQRetrieval": ["train"],
        "NQ-PL": ["train"],  # translation not trained on
        "NQ-NL": ["train"],  # translation not trained on
        # not in MTEB
        # "s2orc": ["train"],
        # "flax-sentence-embeddings/stackexchange_title_body_jsonl": ["train"],
        # "flax-sentence-embeddings/stackexchange_titlebody_best_voted_answer_jsonl": [
        #     "train"
        # ],
        # "flax-sentence-embeddings/stackexchange_title_best_voted_answer_jsonl": [
        #     "train"
        # ],
        # "flax-sentence-embeddings/stackexchange_titlebody_best_and_down_voted_answer_jsonl": [
        #     "train"
        # ],
        # "sentence-transformers/reddit-title-body": ["train"],
        # "msmarco": ["train"],
        # "gooaq": ["train"],
        # "yahoo_answers_topics": ["train"],
        # "code_search_net": ["train"],
        # "search_qa": ["train"],
        # "eli5": ["train"],
        # "snli": ["train"],
        # "multi_nli": ["train"],
        # "wikihow": ["train"],
        # "natural_questions": ["train"],
        # "trivia_qa": ["train"],
        # "embedding-data/sentence-compression": ["train"],
        # "embedding-data/flickr30k-captions": ["train"],
        # "embedding-data/altlex": ["train"],
        # "embedding-data/simple-wiki": ["train"],
        # "embedding-data/QQP": ["train"],
        # "embedding-data/SPECTER": ["train"],
        # "embedding-data/PAQ_pairs": ["train"],
        # "embedding-data/WikiAnswers": ["train"],
        # "sentence-transformers/embedding-training-data": ["train"],
    },
    adapted_from="hum-lodestone-v1",
    superseded_by=None,
)
Jaume__gemma_2b_embeddings = ModelMeta(
    name="Jaume/gemma-2b-embeddings",
    revision="86431f65d7c3f66b2af096c61e614a2958f191f1",
    release_date="2024-06-29",
    languages=[],
    loader=None,
    n_parameters=2506172416,
    memory_usage_mb=9560,
    max_tokens=8192.0,
    embed_dim=2048,
    license=None,
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/Jaume/gemma-2b-embeddings",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets={},
    adapted_from="google/gemma-2b",
    superseded_by=None,
)
BeastyZ__e5_R_mistral_7b = ModelMeta(
    name="BeastyZ/e5-R-mistral-7b",
    revision="3f810a6a7fd220369ad248e3705cf13d71803602",
    release_date="2024-06-28",
    languages=["eng_Latn"],
    loader=None,
    n_parameters=7241732096,
    memory_usage_mb=27625,
    max_tokens=32768.0,
    embed_dim=None,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch"],
    reference="https://huggingface.co/BeastyZ/e5-R-mistral-7b",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=E5_MISTRAL_TRAINING_DATA,
    # not MTEB: {"BeastyZ/E5-R": ["train"]},
    adapted_from="intfloat/e5-mistral-7b-instruct",
    superseded_by=None,
)

bilingual_embedding_training_data = {
    "STSBenchmark": ["train"],
    "STSBenchmarkMultilingualSTS": ["train"],
    "XNLI": ["train"],
    # not in mteb
    # SNLI
}

Lajavaness__bilingual_embedding_base = ModelMeta(
    name="Lajavaness/bilingual-embedding-base",
    revision="0bfc54bb2aa2666dd84715289c7ef58a95eb4d8d",
    release_date="2024-06-26",
    languages=None,
    loader=partial(  # type: ignore
        sentence_transformers_loader,
        model_name="Lajavaness/bilingual-embedding-base",
        revision="0bfc54bb2aa2666dd84715289c7ef58a95eb4d8d",
        trust_remote_code=True,
    ),
    n_parameters=278043648,
    memory_usage_mb=1061,
    max_tokens=514.0,
    embed_dim=768,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/Lajavaness/bilingual-embedding-base",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=bilingual_embedding_training_data,
    adapted_from="dangvantuan/bilingual_impl",
    superseded_by=None,
)
Lajavaness__bilingual_embedding_large = ModelMeta(
    name="Lajavaness/bilingual-embedding-large",
    revision="e83179d7a66e8aed1b3015e98bb5ae234ed89598",
    release_date="2024-06-24",
    languages=["fra_Latn", "eng_Latn"],
    loader=partial(  # type: ignore
        sentence_transformers_loader,
        model_name="Lajavaness/bilingual-embedding-large",
        revision="e83179d7a66e8aed1b3015e98bb5ae234ed89598",
        trust_remote_code=True,
    ),
    n_parameters=559890432,
    memory_usage_mb=2136,
    max_tokens=514.0,
    embed_dim=1024,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/Lajavaness/bilingual-embedding-large",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=bilingual_embedding_training_data,
    adapted_from="dangvantuan/bilingual_impl",
    superseded_by=None,
)
Lajavaness__bilingual_embedding_small = ModelMeta(
    name="Lajavaness/bilingual-embedding-small",
    revision="ed4a1dd814de0db81d4a4e287c296a03194463e3",
    release_date="2024-07-17",
    languages=["fra_Latn", "eng_Latn"],
    loader=partial(  # type: ignore
        sentence_transformers_loader,
        model_name="Lajavaness/bilingual-embedding-small",
        revision="ed4a1dd814de0db81d4a4e287c296a03194463e3",
        trust_remote_code=True,
    ),
    n_parameters=117653760,
    memory_usage_mb=449,
    max_tokens=512.0,
    embed_dim=384,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/Lajavaness/bilingual-embedding-small",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=bilingual_embedding_training_data,
    adapted_from="dangvantuan/bilingual_impl",
    superseded_by=None,
)
Mihaiii__Bulbasaur = ModelMeta(
    name="Mihaiii/Bulbasaur",
    revision="6876f839e18ae36224049a41194a431953f08747",
    release_date="2024-04-27",
    languages=None,
    loader=None,
    n_parameters=17389824,
    memory_usage_mb=66,
    max_tokens=512.0,
    embed_dim=384,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/Mihaiii/Bulbasaur",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=None,  # source model is GTE-tiny where training data is unknown
    # {"Mihaiii/qa-assistant": ["train"]},
    adapted_from="Mihaiii/dwsdwass",
    superseded_by=None,
)
Mihaiii__Ivysaur = ModelMeta(
    name="Mihaiii/Ivysaur",
    revision="65914d976f45beb4bda7485c39d88865b4ce6554",
    release_date="2024-04-27",
    languages=None,
    loader=None,
    n_parameters=22713216,
    memory_usage_mb=87,
    max_tokens=512.0,
    embed_dim=384,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/Mihaiii/Ivysaur",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=None,  # source model is GTE-tiny where training data is unknown
    # not MTEB: {"Mihaiii/qa-assistant": ["train"]},
    adapted_from="Mihaiii/jhjghjgh",
    superseded_by=None,
)
Mihaiii__Squirtle = ModelMeta(
    name="Mihaiii/Squirtle",
    revision="5b991da48a9286637a256d4a35aab87a1a57b78a",
    release_date="2024-04-30",
    languages=None,
    loader=None,
    n_parameters=15615360,
    memory_usage_mb=60,
    max_tokens=512.0,
    embed_dim=384,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/Mihaiii/Squirtle",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=bge_training_data,  # source model is bge-base-en-v1.5
    # not MTEB: {"Mihaiii/qa-assistant": ["train"]},
    adapted_from="Mihaiii/test21",
    superseded_by=None,
)
Mihaiii__Venusaur = ModelMeta(
    name="Mihaiii/Venusaur",
    revision="0dc817f0addbb7bab8feeeeaded538f9ffeb3419",
    release_date="2024-04-29",
    languages=None,
    loader=None,
    n_parameters=15615360,
    memory_usage_mb=60,
    max_tokens=512.0,
    embed_dim=384,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/Mihaiii/Venusaur",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=None,  # source model is unkown
    # {"Mihaiii/qa-assistant": ["train"]},
    adapted_from="Mihaiii/test14",
    superseded_by=None,
)
Mihaiii__Wartortle = ModelMeta(
    name="Mihaiii/Wartortle",
    revision="14caca5253414d38a7d28b62d1b7c30ef3293a87",
    release_date="2024-04-30",
    languages=None,
    loader=None,
    n_parameters=17389824,
    memory_usage_mb=66,
    max_tokens=512.0,
    embed_dim=384,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/Mihaiii/Wartortle",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=bge_training_data,  # distill from  bge-base-en-v1.5
    # {"Mihaiii/qa-assistant": ["train"]},
    adapted_from="Mihaiii/test22",
    superseded_by=None,
)
Mihaiii__gte_micro = ModelMeta(
    name="Mihaiii/gte-micro",
    revision="6fd2397cb9dfa7c901aedf9a2a44d3c888ccafdd",
    release_date="2024-04-21",
    languages=None,
    loader=None,
    n_parameters=17389824,
    memory_usage_mb=66,
    max_tokens=512.0,
    embed_dim=384,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/Mihaiii/gte-micro",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=None,
    adapted_from=None,
    superseded_by=None,
)
Mihaiii__gte_micro_v4 = ModelMeta(
    name="Mihaiii/gte-micro-v4",
    revision="78e1a4b348f8524c3ab2e3e3475788f5adb8c98f",
    release_date="2024-04-22",
    languages=None,
    loader=None,
    n_parameters=19164288,
    memory_usage_mb=73,
    max_tokens=512.0,
    embed_dim=384,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/Mihaiii/gte-micro-v4",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=None,
    adapted_from=None,
    superseded_by=None,
)
OrdalieTech__Solon_embeddings_large_0_1 = ModelMeta(
    name="OrdalieTech/Solon-embeddings-large-0.1",
    revision="9f6465f6ea2f6d10c6294bc15d84edf87d47cdef",
    release_date="2023-12-09",
    languages=["fra_Latn"],
    loader=None,
    n_parameters=559890432,
    memory_usage_mb=2136,
    max_tokens=514.0,
    embed_dim=1024,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch"],
    reference="https://huggingface.co/OrdalieTech/Solon-embeddings-large-0.1",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=None,
    adapted_from="solon-large-06-BIG",
    superseded_by=None,
)
Omartificial_Intelligence_Space__Arabert_all_nli_triplet_Matryoshka = ModelMeta(
    name="Omartificial-Intelligence-Space/Arabert-all-nli-triplet-Matryoshka",
    revision="d0361a36f6fe69febfc8550d0918abab174f6f30",
    release_date="2024-06-16",
    languages=["ara_Arab"],
    loader=None,
    n_parameters=135193344,
    memory_usage_mb=516,
    max_tokens=512.0,
    embed_dim=768,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/Omartificial-Intelligence-Space/Arabert-all-nli-triplet-Matryoshka",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets={},  # not in MTEB: {"Omartificial-Intelligence-Space/Arabic-NLi-Triplet": ["train"]},
    adapted_from="aubmindlab/bert-base-arabertv02",
    superseded_by=None,
)
Omartificial_Intelligence_Space__Arabic_MiniLM_L12_v2_all_nli_triplet = ModelMeta(
    name="Omartificial-Intelligence-Space/Arabic-MiniLM-L12-v2-all-nli-triplet",
    revision="6916465c43b984e955aa6dc72851474f0128f428",
    release_date="2024-06-25",
    languages=["ara_Arab"],
    loader=None,
    n_parameters=117653760,
    memory_usage_mb=449,
    max_tokens=512.0,
    embed_dim=384,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/Omartificial-Intelligence-Space/Arabic-MiniLM-L12-v2-all-nli-triplet",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=sent_trf_training_dataset,
    # not in MTEB
    # {"Omartificial-Intelligence-Space/Arabic-NLi-Triplet": ["train"]},
    adapted_from="sentence-transformers/paraphrase-multilingual-MiniLM-L12-v2",
    superseded_by=None,
)
Omartificial_Intelligence_Space__Arabic_all_nli_triplet_Matryoshka = ModelMeta(
    name="Omartificial-Intelligence-Space/Arabic-all-nli-triplet-Matryoshka",
    revision="1ca467cc576bd76666a4d21b24ee43afa914dd10",
    release_date="2024-06-14",
    languages=["ara_Arab"],
    loader=None,
    n_parameters=278043648,
    memory_usage_mb=1061,
    max_tokens=514.0,
    embed_dim=768,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/Omartificial-Intelligence-Space/Arabic-all-nli-triplet-Matryoshka",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=sent_trf_training_dataset,  # derived from
    # not in MTEB:
    # {"Omartificial-Intelligence-Space/Arabic-NLi-Triplet": ["train"]},
    adapted_from="sentence-transformers/paraphrase-multilingual-mpnet-base-v2",
    superseded_by=None,
)
Omartificial_Intelligence_Space__Arabic_labse_Matryoshka = ModelMeta(
    name="Omartificial-Intelligence-Space/Arabic-labse-Matryoshka",
    revision="ee6d5e33c78ed582ade47fd452a74ea52aa5bfe2",
    release_date="2024-06-16",
    languages=["ara_Arab"],
    loader=None,
    n_parameters=470926848,
    memory_usage_mb=1796,
    max_tokens=512.0,
    embed_dim=768,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/Omartificial-Intelligence-Space/Arabic-labse-Matryoshka",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=None,  # derived from labSE
    # as well as:
    # {"Omartificial-Intelligence-Space/Arabic-NLi-Triplet": ["train"]},
    adapted_from="sentence-transformers/LaBSE",
    superseded_by=None,
)
Omartificial_Intelligence_Space__Arabic_mpnet_base_all_nli_triplet = ModelMeta(
    name="Omartificial-Intelligence-Space/Arabic-mpnet-base-all-nli-triplet",
    revision="2628cb641e040f44328195fadcdfb58e6d5cffa7",
    release_date="2024-06-15",
    languages=["ara_Arab"],
    loader=None,
    n_parameters=109486464,
    memory_usage_mb=418,
    max_tokens=514.0,
    embed_dim=768,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/Omartificial-Intelligence-Space/Arabic-mpnet-base-all-nli-triplet",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=sent_trf_training_dataset,
    # not in MTEB:
    # {"Omartificial-Intelligence-Space/Arabic-NLi-Triplet": ["train"]},
    adapted_from="tomaarsen/mpnet-base-all-nli-triplet",
    superseded_by=None,
)
Omartificial_Intelligence_Space__Marbert_all_nli_triplet_Matryoshka = ModelMeta(
    name="Omartificial-Intelligence-Space/Marbert-all-nli-triplet-Matryoshka",
    revision="ecf3274e164f057c4a3dd70691cae0265d87a9d0",
    release_date="2024-06-17",
    languages=["ara_Arab"],
    loader=None,
    n_parameters=162841344,
    memory_usage_mb=621,
    max_tokens=512.0,
    embed_dim=768,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/Omartificial-Intelligence-Space/Marbert-all-nli-triplet-Matryoshka",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets={},  # not in MTEB: "Omartificial-Intelligence-Space/Arabic-NLi-Triplet": ["train"]},
    adapted_from="UBC-NLP/MARBERTv2",
    superseded_by=None,
)
consciousAI__cai_lunaris_text_embeddings = ModelMeta(
    name="consciousAI/cai-lunaris-text-embeddings",
    revision="8332c464d13505968ff7a6e2213f36fd8730b4c7",
    release_date="2023-06-22",
    languages=None,
    loader=None,
    n_parameters=None,
    memory_usage_mb=None,
    max_tokens=512.0,
    embed_dim=1024,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch"],
    reference="https://huggingface.co/consciousAI/cai-lunaris-text-embeddings",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=None,
    adapted_from="/root/.cache/torch/sentence_transformers/intfloat_e5-large-v2",
    superseded_by=None,
)
consciousAI__cai_stellaris_text_embeddings = ModelMeta(
    name="consciousAI/cai-stellaris-text-embeddings",
    revision="c000ec4b29588daf0f4a0b2ad4e72ee807d8efc0",
    release_date="2023-06-23",
    languages=None,
    loader=None,
    n_parameters=None,
    memory_usage_mb=None,
    max_tokens=514.0,
    embed_dim=768,
    license=None,
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch"],
    reference="https://huggingface.co/consciousAI/cai-stellaris-text-embeddings",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=None,
    adapted_from="/root/.cache/torch/sentence_transformers/sentence-transformers_all-mpnet-base-v1/",
    superseded_by=None,
)

SENTENCE_CROISSANT_TRAINING_DATA = {
    "STS12": ["train"],
    "STSBenchmark": ["train"],
    "STSBenchmarkMultilingualSTS": ["train"],
    "QuoraRetrieval": ["train"],
    "MSMARCO": ["train"],
    "STSB": ["train"],
}
manu__sentence_croissant_alpha_v0_2 = ModelMeta(
    name="manu/sentence_croissant_alpha_v0.2",
    revision="4610b8cea65d7dd59e0b04af50753933fe5b29b2",
    release_date="2024-03-15",
    languages=None,
    loader=None,
    n_parameters=1279887360,
    memory_usage_mb=2441,
    max_tokens=2048.0,
    embed_dim=2048,
    license=None,
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/manu/sentence_croissant_alpha_v0.2",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=SENTENCE_CROISSANT_TRAINING_DATA,
    adapted_from="croissantllm/CroissantCool",
    superseded_by="manu/sentence_croissant_alpha_v0.3",
)
manu__sentence_croissant_alpha_v0_3 = ModelMeta(
    name="manu/sentence_croissant_alpha_v0.3",
    revision="4ac16754f3d81aba76cc32955dc9ee4122df96eb",
    release_date="2024-04-26",
    languages=None,
    loader=None,
    n_parameters=1279887360,
    memory_usage_mb=2441,
    max_tokens=2048.0,
    embed_dim=2048,
    license=None,
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/manu/sentence_croissant_alpha_v0.3",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=SENTENCE_CROISSANT_TRAINING_DATA,
    adapted_from="croissantllm/CroissantCool-v0.2",
    superseded_by="manu/sentence_croissant_alpha_v0.4",
)
manu__sentence_croissant_alpha_v0_4 = ModelMeta(
    name="manu/sentence_croissant_alpha_v0.4",
    revision="0ce6372e6a3c21134dcf26dcde13cca869c767fc",
    release_date="2024-04-27",
    languages=["fra_Latn", "eng_Latn"],
    loader=None,
    n_parameters=1279887360,
    memory_usage_mb=2441,
    max_tokens=2048.0,
    embed_dim=2048,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/manu/sentence_croissant_alpha_v0.4",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=SENTENCE_CROISSANT_TRAINING_DATA,
    # Not in MTEB: {"manu/embedding_data_v2_100k": ["train"]},
    adapted_from="croissantllm/CroissantCool-v0.2",
    superseded_by=None,
)
thenlper__gte_base = ModelMeta(
    name="thenlper/gte-base",
    revision="c078288308d8dee004ab72c6191778064285ec0c",
    release_date="2023-07-27",
    languages=["eng_Latn"],
    loader=None,
    n_parameters=109482752,
    memory_usage_mb=209,
    max_tokens=512.0,
    embed_dim=768,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch"],
    reference="https://huggingface.co/thenlper/gte-base",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=None,
    adapted_from=None,
    superseded_by=None,
)
thenlper__gte_large = ModelMeta(
    name="thenlper/gte-large",
    revision="4bef63f39fcc5e2d6b0aae83089f307af4970164",
    release_date="2023-07-27",
    languages=["eng_Latn"],
    loader=None,
    n_parameters=335142400,
    memory_usage_mb=639,
    max_tokens=512.0,
    embed_dim=1024,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch"],
    reference="https://huggingface.co/thenlper/gte-large",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=None,
    adapted_from=None,
    superseded_by=None,
)
thenlper__gte_small = ModelMeta(
    name="thenlper/gte-small",
    revision="17e1f347d17fe144873b1201da91788898c639cd",
    release_date="2023-07-27",
    languages=["eng_Latn"],
    loader=None,
    n_parameters=33360512,
    memory_usage_mb=64,
    max_tokens=512.0,
    embed_dim=384,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch"],
    reference="https://huggingface.co/thenlper/gte-small",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=None,
    adapted_from=None,
    superseded_by=None,
)
OrlikB__KartonBERT_USE_base_v1 = ModelMeta(
    name="OrlikB/KartonBERT-USE-base-v1",
    revision="1f59dd58fe57995c0e867d5e29f03763eae99645",
    release_date="2024-09-30",
    languages=["pol_Latn"],
    loader=None,
    n_parameters=103705344,
    memory_usage_mb=396,
    max_tokens=512.0,
    embed_dim=768,
    license="gpl-3.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch"],
    reference="https://huggingface.co/OrlikB/KartonBERT-USE-base-v1",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=None,
    adapted_from="KartonBERT-USE-base-v1",
    superseded_by=None,
)
OrlikB__st_polish_kartonberta_base_alpha_v1 = ModelMeta(
    name="OrlikB/st-polish-kartonberta-base-alpha-v1",
    revision="5590a0e2d7bb43674e44d7076b3ff157f7d4a1cb",
    release_date="2023-11-12",
    languages=["pol_Latn"],
    loader=None,
    n_parameters=None,
    memory_usage_mb=None,
    max_tokens=514.0,
    embed_dim=768,
    license="lgpl",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch"],
    reference="https://huggingface.co/OrlikB/st-polish-kartonberta-base-alpha-v1",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=None,
    adapted_from="st-polish-kartonberta-base-alpha-v1",
    superseded_by=None,
)
sdadas__mmlw_e5_base = ModelMeta(
    name="sdadas/mmlw-e5-base",
    revision="f10628ed55b5ec400502aff439bd714a6da0af30",
    release_date="2023-11-17",
    languages=["pol_Latn"],
    loader=None,
    n_parameters=278043648,
    memory_usage_mb=1061,
    max_tokens=514.0,
    embed_dim=768,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch"],
    reference="https://huggingface.co/sdadas/mmlw-e5-base",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=E5_TRAINING_DATA,
    adapted_from="intfloat/multilingual-e5-base",
    superseded_by=None,
)
dwzhu__e5_base_4k = ModelMeta(
    name="dwzhu/e5-base-4k",
    revision="1b5664b8cb2bccd8c309429b7bfe5864402e8fbc",
    release_date="2024-03-28",
    languages=["eng_Latn"],
    loader=None,
    n_parameters=None,
    memory_usage_mb=None,
    max_tokens=4096.0,
    embed_dim=None,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch"],
    reference="https://huggingface.co/dwzhu/e5-base-4k",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets={
        **E5_TRAINING_DATA,
    },
    adapted_from="intfloat/e5-base-v2",
    superseded_by=None,
)
sdadas__mmlw_e5_large = ModelMeta(
    name="sdadas/mmlw-e5-large",
    revision="5c143fb045ebed664fd85b43fc45155999eb110f",
    release_date="2023-11-17",
    languages=["pol_Latn"],
    loader=None,
    n_parameters=559890432,
    memory_usage_mb=2136,
    max_tokens=514.0,
    embed_dim=1024,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch"],
    reference="https://huggingface.co/sdadas/mmlw-e5-large",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=E5_TRAINING_DATA,
    adapted_from="intfloat/multilingual-e5-large",
    superseded_by=None,
)
sdadas__mmlw_e5_small = ModelMeta(
    name="sdadas/mmlw-e5-small",
    revision="ff1298cb6d997f18b794d2f3d73cad2ba2ad739a",
    release_date="2023-11-17",
    languages=["pol_Latn"],
    loader=None,
    n_parameters=117653760,
    memory_usage_mb=449,
    max_tokens=512.0,
    embed_dim=384,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch"],
    reference="https://huggingface.co/sdadas/mmlw-e5-small",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=E5_TRAINING_DATA,
    adapted_from="intfloat/multilingual-e5-small",
    superseded_by=None,
)
sdadas__mmlw_roberta_base = ModelMeta(
    name="sdadas/mmlw-roberta-base",
    revision="0ac7f23f6c96af601fa6a17852bd08d5136d6365",
    release_date="2023-11-17",
    languages=["pol_Latn"],
    loader=None,
    n_parameters=124442880,
    memory_usage_mb=475,
    max_tokens=514.0,
    embed_dim=768,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch"],
    reference="https://huggingface.co/sdadas/mmlw-roberta-base",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets={"MSMARCO": ["train"]},
    adapted_from="sdadas/polish-roberta-base-v2",
    superseded_by=None,
)
sdadas__mmlw_roberta_large = ModelMeta(
    name="sdadas/mmlw-roberta-large",
    revision="b8058066a8de32d0737b3cd82d8b4f4108745af9",
    release_date="2023-11-17",
    languages=["pol_Latn"],
    loader=None,
    n_parameters=434961408,
    memory_usage_mb=1659,
    max_tokens=514.0,
    embed_dim=1024,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch"],
    reference="https://huggingface.co/sdadas/mmlw-roberta-large",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets={"MSMARCO": ["train"]},
    adapted_from="sdadas/polish-roberta-large-v2",
    superseded_by=None,
)
izhx__udever_bloom_1b1 = ModelMeta(
    name="izhx/udever-bloom-1b1",
    revision="7bf1ee29878cb040b2708a691aa4b61f27eaa252",
    release_date="2023-10-24",
    languages=[
        "aka_Latn",
        "ara_Arab",
        "asm_Beng",
        "bam_Latn",
        "ben_Beng",
        "cat_Latn",
        "eng_Latn",
        "spa_Latn",
        "eus_Latn",
        "fon_Latn",
        "fra_Latn",
        "guj_Gujr",
        "hin_Deva",
        "ind_Latn",
        "ibo_Latn",
        "kik_Latn",
        "kan_Knda",
        "lug_Latn",
        "lin_Latn",
        "mal_Mlym",
        "mar_Deva",
        "nep_Deva",
        "nso_Latn",
        "nya_Latn",
        "ori_Orya",
        "pan_Guru",
        "por_Latn",
        "run_Latn",
        "kin_Latn",
        "sna_Latn",
        "sot_Latn",
        "swa_Latn",
        "tam_Taml",
        "tel_Telu",
        "tsn_Latn",
        "tso_Latn",
        "tum_Latn",
        "twi_Latn",
        "urd_Arab",
        "vie_Latn",
        "wol_Latn",
        "xho_Latn",
        "yor_Latn",
        "zho_Hans",
        "zul_Latn",
    ],
    loader=None,
    n_parameters=None,
    memory_usage_mb=None,
    max_tokens=None,
    embed_dim=None,
    license="bigscience-bloom-rail-1.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch"],
    reference="https://huggingface.co/izhx/udever-bloom-1b1",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=None,
    adapted_from="bigscience/bloom-1b1",
    superseded_by=None,
)
izhx__udever_bloom_3b = ModelMeta(
    name="izhx/udever-bloom-3b",
    revision="4edd8affe80ca89ba0f6b6ba4103fc7f25fc57b2",
    release_date="2023-10-24",
    languages=[
        "aka_Latn",
        "ara_Arab",
        "asm_Beng",
        "bam_Latn",
        "ben_Beng",
        "cat_Latn",
        "eng_Latn",
        "spa_Latn",
        "eus_Latn",
        "fon_Latn",
        "fra_Latn",
        "guj_Gujr",
        "hin_Deva",
        "ind_Latn",
        "ibo_Latn",
        "kik_Latn",
        "kan_Knda",
        "lug_Latn",
        "lin_Latn",
        "mal_Mlym",
        "mar_Deva",
        "nep_Deva",
        "nso_Latn",
        "nya_Latn",
        "ori_Orya",
        "pan_Guru",
        "por_Latn",
        "run_Latn",
        "kin_Latn",
        "sna_Latn",
        "sot_Latn",
        "swa_Latn",
        "tam_Taml",
        "tel_Telu",
        "tsn_Latn",
        "tso_Latn",
        "tum_Latn",
        "twi_Latn",
        "urd_Arab",
        "vie_Latn",
        "wol_Latn",
        "xho_Latn",
        "yor_Latn",
        "zho_Hans",
        "zul_Latn",
    ],
    loader=None,
    n_parameters=None,
    memory_usage_mb=None,
    max_tokens=None,
    embed_dim=None,
    license="bigscience-bloom-rail-1.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch"],
    reference="https://huggingface.co/izhx/udever-bloom-3b",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=None,
    adapted_from="bigscience/bloom-3b",
    superseded_by=None,
)
izhx__udever_bloom_560m = ModelMeta(
    name="izhx/udever-bloom-560m",
    revision="b2a723e355946ec5a5c5fbed3459766627ded2bb",
    release_date="2023-10-24",
    languages=[
        "aka_Latn",
        "ara_Arab",
        "asm_Beng",
        "bam_Latn",
        "ben_Beng",
        "cat_Latn",
        "eng_Latn",
        "spa_Latn",
        "eus_Latn",
        "fon_Latn",
        "fra_Latn",
        "guj_Gujr",
        "hin_Deva",
        "ind_Latn",
        "ibo_Latn",
        "kik_Latn",
        "kan_Knda",
        "lug_Latn",
        "lin_Latn",
        "mal_Mlym",
        "mar_Deva",
        "nep_Deva",
        "nso_Latn",
        "nya_Latn",
        "ori_Orya",
        "pan_Guru",
        "por_Latn",
        "run_Latn",
        "kin_Latn",
        "sna_Latn",
        "sot_Latn",
        "swa_Latn",
        "tam_Taml",
        "tel_Telu",
        "tsn_Latn",
        "tso_Latn",
        "tum_Latn",
        "twi_Latn",
        "urd_Arab",
        "vie_Latn",
        "wol_Latn",
        "xho_Latn",
        "yor_Latn",
        "zho_Hans",
        "zul_Latn",
    ],
    loader=None,
    n_parameters=None,
    memory_usage_mb=None,
    max_tokens=None,
    embed_dim=None,
    license="bigscience-bloom-rail-1.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch"],
    reference="https://huggingface.co/izhx/udever-bloom-560m",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=None,
    adapted_from="bigscience/bloom-560m",
    superseded_by=None,
)
izhx__udever_bloom_7b1 = ModelMeta(
    name="izhx/udever-bloom-7b1",
    revision="18e8d3e6dbd94868584877f2e72a105a17df22ef",
    release_date="2023-10-24",
    languages=[
        "aka_Latn",
        "ara_Arab",
        "asm_Beng",
        "bam_Latn",
        "ben_Beng",
        "cat_Latn",
        "eng_Latn",
        "spa_Latn",
        "eus_Latn",
        "fon_Latn",
        "fra_Latn",
        "guj_Gujr",
        "hin_Deva",
        "ind_Latn",
        "ibo_Latn",
        "kik_Latn",
        "kan_Knda",
        "lug_Latn",
        "lin_Latn",
        "mal_Mlym",
        "mar_Deva",
        "nep_Deva",
        "nso_Latn",
        "nya_Latn",
        "ori_Orya",
        "pan_Guru",
        "por_Latn",
        "run_Latn",
        "kin_Latn",
        "sna_Latn",
        "sot_Latn",
        "swa_Latn",
        "tam_Taml",
        "tel_Telu",
        "tsn_Latn",
        "tso_Latn",
        "tum_Latn",
        "twi_Latn",
        "urd_Arab",
        "vie_Latn",
        "wol_Latn",
        "xho_Latn",
        "yor_Latn",
        "zho_Hans",
        "zul_Latn",
    ],
    loader=None,
    n_parameters=None,
    memory_usage_mb=None,
    max_tokens=None,
    embed_dim=None,
    license="bigscience-bloom-rail-1.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch"],
    reference="https://huggingface.co/izhx/udever-bloom-7b1",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=None,
    adapted_from="bigscience/bloom-7b1",
    superseded_by=None,
)
avsolatorio__GIST_Embedding_v0 = ModelMeta(
    name="avsolatorio/GIST-Embedding-v0",
    revision="bf6b2e55e92f510a570ad4d7d2da2ec8cd22590c",
    release_date="2024-01-31",
    languages=["eng_Latn"],
    loader=None,
    n_parameters=109482240,
    memory_usage_mb=418,
    max_tokens=512.0,
    embed_dim=768,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/avsolatorio/GIST-Embedding-v0",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets={
        **bge_training_data,
        # not in mteb:
        # MEDI
        # all MTEB CLF datasets that has a train split:
        "AmazonPolarityClassification": ["train"],
        "AmazonReviewsClassification": ["train"],
        "EmotionClassification": ["train"],
        "ImdbClassification": ["train"],
        "MTOPDomainClassification": ["train"],
        "MTOPIntentClassification": ["train"],
        "MassiveIntentClassification": ["train"],
        "MassiveScenarioClassification": ["train"],
        "ToxicConversationsClassification": ["train"],
        "TweetSentimentExtractionClassification": ["train"],
        "Banking77Classification": ["train"],
        "AmazonCounterfactualClassification": ["train"],
    },
    adapted_from="BAAI/bge-large-en-v1.5",
    superseded_by=None,
)
avsolatorio__GIST_all_MiniLM_L6_v2 = ModelMeta(
    name="avsolatorio/GIST-all-MiniLM-L6-v2",
    revision="ea89dfad053bba14677bb784a4269898abbdce44",
    release_date="2024-02-03",
    languages=["eng_Latn"],
    loader=None,
    n_parameters=22713216,
    memory_usage_mb=87,
    max_tokens=512.0,
    embed_dim=384,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/avsolatorio/GIST-all-MiniLM-L6-v2",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets={
        **bge_training_data,
        # not in mteb:
        # MEDI
        # all MTEB CLF datasets that has a train split:
        "AmazonPolarityClassification": ["train"],
        "AmazonReviewsClassification": ["train"],
        "EmotionClassification": ["train"],
        "ImdbClassification": ["train"],
        "MTOPDomainClassification": ["train"],
        "MTOPIntentClassification": ["train"],
        "MassiveIntentClassification": ["train"],
        "MassiveScenarioClassification": ["train"],
        "ToxicConversationsClassification": ["train"],
        "TweetSentimentExtractionClassification": ["train"],
        "Banking77Classification": ["train"],
        "AmazonCounterfactualClassification": ["train"],
    },
    adapted_from=None,
    superseded_by=None,
)
avsolatorio__GIST_large_Embedding_v0 = ModelMeta(
    name="avsolatorio/GIST-large-Embedding-v0",
    revision="7831200e2f7819b994490c091cf3258a2b821f0c",
    release_date="2024-02-14",
    languages=["eng_Latn"],
    loader=None,
    n_parameters=335141888,
    memory_usage_mb=1278,
    max_tokens=512.0,
    embed_dim=1024,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/avsolatorio/GIST-large-Embedding-v0",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets={
        **bge_training_data,
        # not in mteb:
        # MEDI
        # all MTEB CLF datasets that has a train split:
        "AmazonPolarityClassification": ["train"],
        "AmazonReviewsClassification": ["train"],
        "EmotionClassification": ["train"],
        "ImdbClassification": ["train"],
        "MTOPDomainClassification": ["train"],
        "MTOPIntentClassification": ["train"],
        "MassiveIntentClassification": ["train"],
        "MassiveScenarioClassification": ["train"],
        "ToxicConversationsClassification": ["train"],
        "TweetSentimentExtractionClassification": ["train"],
        "Banking77Classification": ["train"],
        "AmazonCounterfactualClassification": ["train"],
    },
    adapted_from=None,
    superseded_by=None,
)
avsolatorio__GIST_small_Embedding_v0 = ModelMeta(
    name="avsolatorio/GIST-small-Embedding-v0",
    revision="d6c4190f9e01b9994dc7cac99cf2f2b85cfb57bc",
    release_date="2024-02-03",
    languages=["eng_Latn"],
    loader=None,
    n_parameters=33360000,
    memory_usage_mb=127,
    max_tokens=512.0,
    embed_dim=384,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/avsolatorio/GIST-small-Embedding-v0",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets={
        **bge_training_data,
        # not in mteb:
        # MEDI
        # all MTEB CLF datasets that has a train split:
        "AmazonPolarityClassification": ["train"],
        "AmazonReviewsClassification": ["train"],
        "EmotionClassification": ["train"],
        "ImdbClassification": ["train"],
        "MTOPDomainClassification": ["train"],
        "MTOPIntentClassification": ["train"],
        "MassiveIntentClassification": ["train"],
        "MassiveScenarioClassification": ["train"],
        "ToxicConversationsClassification": ["train"],
        "TweetSentimentExtractionClassification": ["train"],
        "Banking77Classification": ["train"],
        "AmazonCounterfactualClassification": ["train"],
    },
    adapted_from=None,
    superseded_by=None,
)
bigscience__sgpt_bloom_7b1_msmarco = ModelMeta(
    name="bigscience/sgpt-bloom-7b1-msmarco",
    revision="dc579f3d2d5a0795eba2049e16c3e36c74007ad3",
    release_date="2022-08-26",
    languages=None,
    loader=None,
    n_parameters=None,
    memory_usage_mb=None,
    max_tokens=None,
    embed_dim=4096,
    license=None,
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch"],
    reference="https://huggingface.co/bigscience/sgpt-bloom-7b1-msmarco",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=None,
    adapted_from="/gpfsscratch/rech/six/commun/commun/experiments/muennighoff/bloomckpt/6b3/bloom-7b1",
    superseded_by=None,
)
aari1995__German_Semantic_STS_V2 = ModelMeta(
    name="aari1995/German_Semantic_STS_V2",
    revision="22912542b0ec7a7ef369837e28ffe6352a27afc9",
    release_date="2022-11-17",
    languages=["deu_Latn"],
    loader=None,
    n_parameters=335736320,
    memory_usage_mb=1281,
    max_tokens=512.0,
    embed_dim=1024,
    license=None,
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch"],
    reference="https://huggingface.co/aari1995/German_Semantic_STS_V2",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=None,  # couldn't figure out the source model
    # {"stsb_multi_mt": ["train"]},
    adapted_from="/content/drive/MyDrive/Stanford_NLU/Project/false_friends/gbert_large_sts_only",
    superseded_by=None,
)
abhinand__MedEmbed_small_v0_1 = ModelMeta(
    name="abhinand/MedEmbed-small-v0.1",
    revision="40a5850d046cfdb56154e332b4d7099b63e8d50e",
    release_date="2024-10-20",
    languages=["eng_Latn"],
    loader=None,
    n_parameters=33360000,
    memory_usage_mb=127,
    max_tokens=512.0,
    embed_dim=384,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch"],
    reference="https://huggingface.co/abhinand/MedEmbed-small-v0.1",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets={
        "MedicalQARetrieval": ["train"],
        "NFCorpus": ["train"],
        "NFCorpus-NL": ["train"],  # translation not trained on
        "PublicHealthQA": ["train"],
        "TRECCOVID": ["train"],
        "ArguAna": ["train"],
        "TRECCOVID-NL": ["train"],  # translation not trained on
        "ArguAna-NL": ["train"],  # translation not trained on
    },
    adapted_from="./medical-bge-small-v1-mix1",
    superseded_by=None,
)
avsolatorio__NoInstruct_small_Embedding_v0 = ModelMeta(
    name="avsolatorio/NoInstruct-small-Embedding-v0",
    revision="b38747000553d8268915c95a55fc87e707c9aadd",
    release_date="2024-05-01",
    languages=["eng_Latn"],
    loader=None,
    n_parameters=33360000,
    memory_usage_mb=127,
    max_tokens=512.0,
    embed_dim=384,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/avsolatorio/NoInstruct-small-Embedding-v0",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=None,
    adapted_from=None,
    superseded_by=None,
)
brahmairesearch__slx_v0_1 = ModelMeta(
    name="brahmairesearch/slx-v0.1",
    revision="688c83fd1a7f34b25575a2bc26cfd87c11b4ce71",
    release_date="2024-08-13",
    languages=["eng_Latn"],
    loader=None,
    n_parameters=22713216,
    memory_usage_mb=87,
    max_tokens=512.0,
    embed_dim=384,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/brahmairesearch/slx-v0.1",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=None,
    adapted_from=None,
    superseded_by=None,
)
deepfile__embedder_100p = ModelMeta(
    name="deepfile/embedder-100p",
    revision="aa02f08f11517977fbcdc94dc9dbf9a1ca152d9b",
    release_date="2023-07-24",
    languages=None,
    loader=None,
    n_parameters=None,
    memory_usage_mb=1061,
    max_tokens=514.0,
    embed_dim=768,
    license=None,
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch"],
    reference="https://huggingface.co/deepfile/embedder-100p",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=None,
    adapted_from="sentence-transformers/paraphrase-multilingual-mpnet-base-v2",
    superseded_by=None,
)
deepvk__USER_bge_m3 = ModelMeta(
    name="deepvk/USER-bge-m3",
    revision="0cc6cfe48e260fb0474c753087a69369e88709ae",
    release_date="2024-07-05",
    languages=["rus_Cyrl"],
    loader=None,
    n_parameters=359026688,
    memory_usage_mb=1370,
    max_tokens=8194.0,
    embed_dim=1024,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/deepvk/USER-bge-m3",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=bge_m3_training_data,  # derived from.
    # not in MTEB:
    # "deepvk/ru-HNP": ["train"],
    # "deepvk/ru-WANLI": ["train"],
    # "Shitao/bge-m3-data": ["train"],
    # "RussianNLP/russian_super_glue": ["train"],
    # "reciTAL/mlsum": ["train"],
    # "Milana/russian_keywords": ["train"],
    # "IlyaGusev/gazeta": ["train"],
    # "d0rj/gsm8k-ru": ["train"],
    # "bragovo/dsum_ru": ["train"],
    # "CarlBrendt/Summ_Dialog_News": ["train"],
    adapted_from="USER-bge-m3",
    superseded_by=None,
)
infgrad__stella_base_en_v2 = ModelMeta(
    name="infgrad/stella-base-en-v2",
    revision="c9e80ff9892d80b39dc54e30a7873f91ea161034",
    release_date="2023-10-19",
    languages=["eng_Latn"],
    loader=None,
    n_parameters=None,
    memory_usage_mb=None,
    max_tokens=512.0,
    embed_dim=None,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch"],
    reference="https://huggingface.co/infgrad/stella-base-en-v2",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=None,
    adapted_from=None,
    superseded_by=None,
)
malenia1__ternary_weight_embedding = ModelMeta(
    name="malenia1/ternary-weight-embedding",
    revision="a1208fb7f646647bb62639fd2e1eb6cc2ef3738e",
    release_date="2024-10-23",
    languages=None,
    loader=None,
    n_parameters=98688000,
    memory_usage_mb=158,
    max_tokens=512.0,
    embed_dim=1024,
    license=None,
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch"],
    reference="https://huggingface.co/malenia1/ternary-weight-embedding",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=None,
    adapted_from="ternary-weight-embedding",
    superseded_by=None,
)
omarelshehy__arabic_english_sts_matryoshka = ModelMeta(
    name="omarelshehy/arabic-english-sts-matryoshka",
    revision="763d116fbe8bf7883c64635c862feeaa3768bb64",
    release_date="2024-10-13",
    languages=["ara_Arab", "eng_Latn"],
    loader=None,
    n_parameters=559890432,
    memory_usage_mb=2136,
    max_tokens=514.0,
    embed_dim=1024,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/omarelshehy/arabic-english-sts-matryoshka",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=None,
    adapted_from="FacebookAI/xlm-roberta-large",
    superseded_by=None,
)
openbmb__MiniCPM_Embedding = ModelMeta(
    loader=partial(  # type: ignore
        sentence_transformers_loader,
        model_name="openbmb/MiniCPM-Embedding",
        revision="c0cb2de33fb366e17c30f9d53142ff11bc18e049",
        # https://huggingface.co/openbmb/MiniCPM-Embedding/blob/c0cb2de33fb366e17c30f9d53142ff11bc18e049/README.md?code=true#L405
        model_kwargs={
            # "attn_implementation": "flash_attention_2",
            "torch_dtype": torch.float16,
        },
        trust_remote_code=True,
    ),
    name="openbmb/MiniCPM-Embedding",
    revision="c0cb2de33fb366e17c30f9d53142ff11bc18e049",
    release_date="2024-09-04",
    languages=["zho_Hans", "eng_Latn"],
    n_parameters=2724880896,
    memory_usage_mb=5197,
    max_tokens=512.0,
    embed_dim=2304,
    license=None,
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/openbmb/MiniCPM-Embedding",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=None,
    adapted_from=None,
    superseded_by=None,
)
shibing624__text2vec_base_multilingual = ModelMeta(
    name="shibing624/text2vec-base-multilingual",
    revision="6633dc49e554de7105458f8f2e96445c6598e9d1",
    release_date="2023-06-22",
    languages=[
        "zho_Hans",
        "eng_Latn",
        "deu_Latn",
        "fra_Latn",
        "ita_Latn",
        "nld_Latn",
        "por_Latn",
        "pol_Latn",
        "rus_Cyrl",
    ],
    loader=None,
    n_parameters=117654272,
    memory_usage_mb=449,
    max_tokens=512.0,
    embed_dim=384,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/shibing624/text2vec-base-multilingual",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=sent_trf_training_dataset,
    # not MTEB: {"shibing624/nli-zh-all": ["train"]},
    adapted_from="sentence-transformers/paraphrase-multilingual-MiniLM-L12-v2",
    superseded_by=None,
)
silma_ai__silma_embeddding_matryoshka_v0_1 = ModelMeta(
    name="silma-ai/silma-embeddding-matryoshka-v0.1",
    revision="a520977a9542ebdb8a7206df6b7ff6977f1886ea",
    release_date="2024-10-12",
    languages=["ara_Arab", "eng_Latn"],
    loader=None,
    n_parameters=135193344,
    memory_usage_mb=516,
    max_tokens=512.0,
    embed_dim=768,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/silma-ai/silma-embeddding-matryoshka-v0.1",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=None,
    adapted_from="/workspace/v3-matryoshka_aubmindlab-bert-base-arabertv02-2024-10-12_13-55-06/checkpoint-26250",
    superseded_by=None,
)

sbert_chinese_general_v1 = ModelMeta(
    name="DMetaSoul/sbert-chinese-general-v1",
    revision="bd27765956bcc2fcf682de0097819947ac10037e",
    release_date="2022-03-25",
    languages=["zho_Hans"],
    loader=None,
    n_parameters=None,
    memory_usage_mb=None,  # Not visible on repo
    max_tokens=512,
    embed_dim=128,
    license="apache-2",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/DMetaSoul/sbert-chinese-general-v1",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets={
        "PAWSX": ["train"],
        "PawsXPairClassification": ["train"],  # they do not specify which one
        # They might have trained on other datasets too, they don't say:
        # "trained on semantically similar datasets such as NLI, PAWS-X, PKU-Paraphrase-Bank, and STS."
    },
    superseded_by=None,
)
dmeta_embedding_zh_small = ModelMeta(
    name="DMetaSoul/Dmeta-embedding-zh-small",
    revision="2050d3439a2f68999dd648c1697471acaac37a29",
    release_date="2024-03-25",
    languages=["zho_Hans"],
    loader=None,
    n_parameters=74.2 * 1e6,
    memory_usage_mb=283,
    max_tokens=1024,
    embed_dim=768,
    license="apache-2",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/DMetaSoul/Dmeta-embedding-zh-small/",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=None,  # They don't specify
    superseded_by=None,
)
xiaobu_embedding = ModelMeta(
    name="lier007/xiaobu-embedding",
    revision="59c79d82eb5223cd9895f6eb8e825c7fa10e4e92",
    release_date="2024-01-09",
    languages=["zho_Hans"],
    loader=None,
    n_parameters=326 * 1e6,
    memory_usage_mb=1244,
    max_tokens=512,
    embed_dim=1024,
    license="not specified",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/lier007/xiaobu-embedding",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=None,  # Finetuned from GTE, none of them disclose training data
    superseded_by=None,
    adapted_from="thenlper/gte-large-zh",
)
xiaobu_embedding_v2 = ModelMeta(
    name="lier007/xiaobu-embedding-v2",
    revision="1912f2e59a5c2ef802a471d735a38702a5c9485e",
    release_date="2024-06-30",
    languages=["zho_Hans"],
    loader=None,
    n_parameters=326 * 1e6,
    memory_usage_mb=1242,
    max_tokens=512,
    embed_dim=768,
    license="not specified",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/lier007/xiaobu-embedding-v2",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=None,  # Finetuned from piccolo-embedding, none of them say
    superseded_by=None,
    adapted_from="sensenova/piccolo-base-zh",
)
yinka_embedding = ModelMeta(
    name="Classical/Yinka",
    revision="59c79d82eb5223cd9895f6eb8e825c7fa10e4e92",
    release_date="2024-01-09",
    languages=["zho_Hans"],
    loader=None,
    n_parameters=326 * 1e6,
    memory_usage_mb=1244,
    max_tokens=512,
    embed_dim=1024,
    license="not specified",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/Classical/Yinka",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=None,  # Not disclosed
    superseded_by=None,
    adapted_from="dunzhang/stella-mrl-large-zh-v3.5-1792d",
)
conan_embedding = ModelMeta(
    name="TencentBAC/Conan-embedding-v1",
    revision="bb9749a57d4f02fd71722386f8d0f5a9398d7eeb",
    release_date="2024-08-22",
    languages=["zho_Hans"],
    loader=None,
    n_parameters=326 * 1e6,
    memory_usage_mb=1242,
    max_tokens=512,
    embed_dim=768,
    license="cc-by-nc-4.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/Classical/Yinka",
    similarity_fn_name="cosine",
    use_instructions=None,
    # source: https://arxiv.org/pdf/2408.15710
    training_datasets=None,  # They "scraped" things from the internet, we don't know, could be leakage
    superseded_by=None,
)
ember_v1 = ModelMeta(
    name="llmrails/ember-v1",
    revision="5e5ce5904901f6ce1c353a95020f17f09e5d021d",
    release_date="2023-10-10",
    languages=["eng_Latn"],
    n_parameters=335 * 1e6,
    memory_usage_mb=1278,
    max_tokens=512,
    embed_dim=1024,
    license="mit",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/llmrails/ember-v1",
    similarity_fn_name="cosine",
    use_instructions=None,
    training_datasets=None,
    superseded_by=None,
)
amazon_titan_text_embeddings_v2 = ModelMeta(
    name="amazon/Titan-text-embeddings-v2",
    revision="1",
    release_date="2024-04-30",
    languages=["eng_Latn"],
    n_parameters=None,
    memory_usage_mb=None,
    max_tokens=None,
    embed_dim=None,
    license="proprietary",
    open_weights=False,
    public_training_code=None,
    public_training_data=None,
    framework=[],
    reference="https://huggingface.co/amazon/Titan-text-embeddings-v2",
    similarity_fn_name="cosine",
    use_instructions=False,
    training_datasets=None,
    superseded_by=None,
)



from __future__ import annotations

import logging
from collections import Counter
from typing import Any

from datasets import Dataset

from ...encoder_interface import Encoder
from ...evaluation.evaluators import Any2TextMultipleChoiceEvaluator
from ..AbsTask import AbsTask, ScoresDict
from ..TaskMetadata import DescriptiveStatistics

logger = logging.getLogger(__name__)


class Any2TextMutipleChoiceDescriptiveStatistics(DescriptiveStatistics):
    """Descriptive statistics for Any2TextMutipleChoice

    Attributes:
        num_samples: number of samples in the dataset.

        min_image_width: Minimum width of images
        average_image_width: Average width of images
        max_image_width: Maximum width of images

        min_image_height: Minimum height of images
        average_image_height: Average height of images
        max_image_height: Maximum height of images

        min_num_choices: Minimum number of choices
        average_num_choices: Average number of choices
        max_num_choices: Maximum number of choices

        answers: dict of answer frequencies

        min_question_length: Minimum length of questions
        average_question_length: Average length of questions
        max_question_length: Maximum length of questions
    """

    num_samples: int

    min_image_width: float
    average_image_width: float
    max_image_width: float

    min_image_height: float
    average_image_height: float
    max_image_height: float

    min_num_choices: int
    average_num_choices: float
    max_num_choices: int

    answers: dict[str, dict[str, int]]

    min_question_length: int
    average_question_length: float
    max_question_length: int


class AbsTaskAny2TextMultipleChoice(AbsTask):
    """Abstract class for Any to Text Multiple Choice tasks,
    where the queries and be either text or image, or both.
    This task assess interleaved encoding of queries,
    the similarity computed between the queries and the candidate choices is ranked.

    self.load_data() must generate a huggingface dataset with a split matching self.metadata_dict["eval_splits"], and assign it to self.dataset.
    """

    query_modalities: list[str] | str = ["image", "text"]
    query_column_names: dict = {"image": "image", "text": "question"}
    label_column_name: str = "answer"
    choices_column_name: str = "choices"

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def _add_main_score(self, scores) -> None:
        scores["main_score"] = scores[self.metadata.main_score]

    def _calculate_metrics_from_split(
        self, split: str, hf_subset: str | None = None, compute_overall: bool = False
    ) -> Any2TextMutipleChoiceDescriptiveStatistics:
        imgs = self.dataset[split][self.query_column_names["image"]]
        questions = self.dataset[split][self.query_column_names["text"]]
        choices = self.dataset[split][self.choices_column_name]
        answers = self.dataset[split][self.label_column_name]

        num_samples = len(answers)
        answer_count = Counter(answers)
        img_widths, img_heights = [], []
        for img in imgs:
            width, height = img.size
            img_heights.append(height)
            img_widths.append(width)

        choices_len = [len(c) for c in choices]
        questions_len = [len(q) for q in questions]

        return Any2TextMutipleChoiceDescriptiveStatistics(
            num_samples=num_samples,
            min_image_width=min(img_widths),
            average_image_width=sum(img_widths) / len(img_widths),
            max_image_width=max(img_widths),
            min_image_height=min(img_heights),
            average_image_height=sum(img_heights) / len(img_heights),
            max_image_height=max(img_heights),
            min_num_choices=min(choices_len),
            average_num_choices=sum(choices_len) / len(choices_len),
            max_num_choices=max(choices_len),
            min_question_length=min(questions_len),
            average_question_length=sum(questions_len) / len(questions_len),
            max_question_length=max(questions_len),
            answers={
                str(answer): {"count": count} for answer, count in answer_count.items()
            },
        )

    def _evaluate_subset(
        self,
        model: Encoder,
        dataset: Dataset,
        *,
        encode_kwargs: dict[str, Any] = {},
        **kwargs,
    ) -> ScoresDict:
        for modality in self.query_modalities:
            if modality not in self.query_column_names:
                raise KeyError(
                    f"query column name of modality {modality} is not defined"
                )
        evaluator = Any2TextMultipleChoiceEvaluator(
            dataset,
            query_modalities=self.query_modalities,
            query_column_names=self.query_column_names,
            label_column_name=self.label_column_name,
            choices_column_name=self.choices_column_name,
            task_name=self.metadata.name,
            **kwargs,
        )
        scores = evaluator(model, encode_kwargs=encode_kwargs)
        self._add_main_score(scores)
        return scores
